----------------------------------------------------------------------
-- PLL : 
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
library IEEE;
use IEEE.ELECTRICAL_SYSTEMS.all;
use IEEE.MATH_REAL.all;

entity PLL is
  generic(Fpole: REAL := 10.0e+3;
          Fzero: REAL := 1.0e+6;
          Mf: REAL := 6.23e+5;
          Fcenter: REAL := 1.0e+6);
  port(terminal input,pll_lfout,pll_out: ELECTRICAL);
end entity PLL;

architecture behav of PLL is
  quantity v_in across input to electrical_ref;
  quantity v_lf across i_lf through pll_lfout to electrical_ref;
  quantity v_vco across i_vco through pll_out to electrical_ref;
  quantity phi: REAL;
  quantity pd_vout: REAL;
  constant wp: real := math_2_pi*fpole;
  constant wz: real := math_2_pi*fzero;
  constant wc: real := math_2_pi*Fcenter;
begin
  if domain = quiescent_domain use
    phi == 0.0;
  else
-- phase extraction in VCO block
    phi'DOT == wc+Mf*v_lf;
  end use;
-- phase detector block
  pd_vout == v_in*v_vco;
-- loop filter block
  v_lf == pd_vout'LTF((1.0,(1.0/wz)),(1.0,(1.0/wp)));
-- output VCO voltage in VCO block
  v_vco == 1.0*cos(phi);
end architecture behav;