-------------------------------------------------------------------------------
-- electrical - thermal coupling example
--
-- Copyright (C) 2017 JSAE
------------------------------------------------------------------------------
library IEEE ;
    use IEEE.ELECTRICAL_SYSTEMS.all ; 
    use IEEE.THERMAL_SYSTEMS.all ;
--------------------
entity WIRE is
  generic (
    r_nom : RESISTANCE := 100.0e-3 ; 
    R_th  : THERMAL_RESISTANCE := 1.0/0.1 ; 
    C_th  : THERMAL_CAPACITANCE:= 1.0 
  ) ; 
  port(
    terminal EL1, EL2 : ELECTRICAL
  ) ; 
end entity WIRE;
--------------------
architecture ARCH_WIRE of WIRE is
  quantity v across i through EL1 to EL2 ; 
  quantity R : RESISTANCE := r_nom ;
  quantity h : HEAT_FLOW ;
  quantity t : TEMPERATURE := PHYS_CTOK; -- 273.15[K]
begin

  R == r_nom*(1.0+0.0044 *(t - PHYS_CTOK +20.0)) ;

  h == i**2.0 * R ; 

  v == i * R ; 

  h == C_th*t'dot + 1.0/R_th*(t - PHYS_CTOK) ; 

end architecture ARCH_WIRE;
