-------------------------------------------------------------------------------
-- thermal coupling model with external thermal pin
--
-- Copyright (C) 2017 JSAE
------------------------------------------------------------------------------
library IEEE ;
    use IEEE.ELECTRICAL_SYSTEMS.all ; 
    use IEEE.THERMAL_SYSTEMS.all ;
--------------------
entity WIRE_hN is
  generic (
    r_nom : RESISTANCE := 100.0e-3 ; 
    R_th  : THERMAL_RESISTANCE := 1.0/0.1 ; 
    C_th  : THERMAL_CAPACITANCE:= 1.0 
  ) ; 
  port(
    terminal EL1, EL2 : ELECTRICAL ;
    terminal TH1 : THERMAL
  ) ; 
end entity WIRE_hN ;
--------------------
architecture ARCH_WIRE of WIRE_hN is
  quantity v across i through EL1 to EL2 ; 
  quantity tex across hex through TH1 to THERMAL_REF ;
  quantity R : RESISTANCE := r_nom ;
  quantity h : HEAT_FLOW ;
  quantity t : TEMPERATURE := PHYS_CTOK; -- 273.15[K]
begin

  R == r_nom*(1.0+0.0044 *(t - PHYS_CTOK +20.0)) ;

  h == i**2.0 * R ; 

  v == i * R ; 

  h == C_th*t'dot + hex ; 

  hex == 1.0/R_th*(t - tex) ;

end architecture ARCH_WIRE;
