-------------------------------------------------------------------------------
-- battery tutorial
--
-- Copyright (C) 2017 JSAE
------------------------------------------------------------------------------
library IEEE ;
    use IEEE.ELECTRICAL_SYSTEMS.all ;
    use IEEE.MATH_REAL.all ; 

library fundamentals_vda ; 
    use fundamentals_vda.tlu_vda.all ; 
--------------------
entity batt_emf is

  generic ( soc0 : REAL := 100.0 ) ;  -- initial state soc [%]
  port (terminal P, N : ELECTRICAL ) ; 

end entity batt_emf;
--------------------
architecture arch_batt_emf of batt_emf is
  constant FCC : REAL := 30.0 ; -- Battery rating capacitance [Ah]
  constant Rdc : REAL := 6.2e-3 ; -- Battery internal resistance [ohm]

  constant soc_map : REAL_VECTOR := (5.0, 20.0, 80.0, 90.0, 100.0) ; 
  quantity soc : REAL ; 	-- State of Charge [%]

  constant emf_map : REAL_VECTOR := (11.0, 12.0, 12.5, 12.6, 12.88) ; 
  quantity emf : REAL ;   -- Electro motive force [V]
	
  quantity v across i through p to n ; 

begin

  soc == realmax( soc0 + 100.0/(FCC*3600.0)* i'integ, 0.0) ;

  emf == realmax(lookup_1d(soc,  soc_map, emf_map), 0.0) ; 

  v == Rdc * i + emf ;

end architecture arch_batt_emf;

