-------------------------------------------------------------------------------
-- motor efficiency map tutorial
--
-- Copyright (C) 2017 JSAE
------------------------------------------------------------------------------
library IEEE;
    use IEEE.ELECTRICAL_SYSTEMS.all;
    use IEEE.MECHANICAL_SYSTEMS.all;
    use IEEE.MATH_REAL.all;

library fundamentals_vda ; 
    use fundamentals_vda.tlu_vda.all ; 
--------------------
entity avg_motor_unit is
  generic (
    TAU_DRIVE : REAL:= 0.005  -- time constant for torque generation [s]
  );
  port (
    terminal DCp, DCn  : ELECTRICAL;  	-- Connection to power supply
    terminal SHAFT : ROTATIONAL_V ;   	-- connection point to pinion
    quantity ETA_DA : in REAL := 0.97 ; -- inverter losses 
    quantity Tref : in REAL  := 0.0 ;  -- required torque
    quantity N : out REAL   -- mechanical rotation speed [rpm]
  );
end entity avg_motor_unit;
--------------------
architecture behav of avg_motor_unit is
  quantity v across i through DCp to DCn ;  -- power supply
  quantity omega  across load through shaft to ROTATIONAL_V_REF;  -- motor shaft
   
  constant NrpmArr : REAL_VECTOR := 
      (5.5, 1395.9, 2786.3, 4176.6, 5567.0, 6957.4, 8347.8, 9738.2) ;
  constant TrqmArr : REAL_VECTOR := 
      (0.50, 13.06, 25.62, 38.18, 50.74) ; 

  constant MotorEffArr : REAL_VECTOR := (
      62.56, 64.31, 59.12, 54.88, 51.29, 47.99, 45.35, 42.89,  --Trqm[0]
      18.00, 95.66, 95.95, 95.71, 95.35, 95.00, 94.73, 94.26,  --Trqm[1]
      10.97, 95.34, 96.49, 96.67, 96.61, 96.57, 96.34, 95.88,    
       7.93, 94.44, 96.29, 96.77, 96.89, 96.92, 96.41, 96.02,    
       6.16, 93.34, 95.84, 96.57, 96.83,  0.0,   0.0,   0.0    
      ) ; 

  quantity nrpm_arg, trq_arg : REAL ;  
  quantity motorEff : REAL ;
  quantity mi : REAL := 0.0 ; 

begin

  mi == -load ; 
  mi'dot == (Tref - mi) / TAU_DRIVE ;   -- (2.4) 
  N == omega * 30.0/MATH_PI ; 

  nrpm_arg == realmin(
               realmax(N,  NrpmArr(NrpmArr'LEFT)), 
                NrpmArr(NrpmArr'RIGHT)) ;

  trq_arg  == realmin(
               realmax(abs(mi), TrqmArr(TrqmArr'LEFT)), 
                TrqmArr(TrqmArr'RIGHT)) ; 

  motorEff == lookup_2D(nrpm_arg, trq_arg,  
                        NrpmArr, TrqmArr, MotorEffArr)/100.0 ;

  if(mi > 0.0) use
    v * i * ETA_DA*motorEff == omega * mi  ; -- (2.2)
  elsif(mi = 0.0) use 
    i == 0.0 ; 
  else 
    v * i  == omega * mi *ETA_DA*motorEff ;  -- (2.3)
  end use ; 

end architecture behav;
