----------------------------------------------------------------------
-- Q_output_lowpass filter : 
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
library IEEE;
  use IEEE.MATH_REAL.all;
----------------------------------------------------------------------
entity q_LPF_jsae is
  generic (
    Fp : REAL; -- Pole frequency [Hz]
    K : REAL := 1.0; -- Filter gain [No Units]
    Q : REAL := 0.707; -- Quality factor [No Units]
    Fsmp : REAL := 10.0e3 -- For Z-dmn only: Sample frequency [Hz]
  );
  port (
    quantity EL_input : in REAL;
    quantity EL_output : out REAL
  );
end entity q_LPF_jsae;
----------------------------------------------------------------------
-- 1st-order S Domain Implementation
architecture s_dmn_1st of q_LPF_jsae is
  constant wp : REAL := math_2_pi*Fp; -- Frequency in Radians
  constant num : REAL_VECTOR := (wp, 0.0); -- Numerator array
  constant den : REAL_VECTOR := (wp, 1.0); -- Denominator array
begin
  EL_output == K * EL_input'LTF(num, den); -- Laplace Transfer Function
end architecture s_dmn_1st;
----------------------------------------------------------------------
-- 1st-order Z Domain Implementation (via bilinear transform, no pre-warping)
architecture z_dmn_1st of q_LPF_jsae is
  constant Tsmp : REAL := 1.0/Fsmp; -- Sample period
  constant wp : REAL := fp*math_2_pi; -- Pole in rad/s
  constant numz_0 : REAL := Tsmp*wp; -- z0 numerator coef??cient
  constant numz_1 : REAL := Tsmp*wp; -- z-1 numerator coef??cient
  constant denz_0 : REAL := Tsmp*wp + 2.0; -- z0 denominator coef??cient
  constant denz_1 : REAL := Tsmp*wp - 2.0; -- z-1 denominator coef??cient
  constant num : REAL_VECTOR := (numz_0, numz_1);
  constant den : REAL_VECTOR := (denz_0, denz_1);
begin -- ZTF
  EL_output == K*EL_input'ZTF(num, den, Tsmp);
end z_dmn_1st;
----------------------------------------------------------------------
 -- 2nd-order S Domain Implementation
architecture s_dmn_2nd of q_LPF_jsae is
  constant wp : REAL := math_2_pi*Fp; -- Frequency in Radians
  constant num : REAL_VECTOR := (wp*wp, 0.0, 0.0); -- Numerator array
  constant den : REAL_VECTOR := (wp*wp, wp/Q, 1.0); -- Denominator array

begin
  EL_output == K * EL_input'LTF(num, den); -- Laplace Transfer Function
end architecture s_dmn_2nd;
----------------------------------------------------------------------
 -- 2nd-order Z Domain Implementation (via bilinear transform, no pre-warping)
architecture z_dmn_2nd of q_LPF_jsae is
  constant T : REAL := 1.0/Fsmp; -- Sample period
  constant wp : REAL := math_2_pi*Fp; -- Frequency in Radians
  constant n0 : REAL := Q*T**3*wp**2; -- num z-0
  constant n1 : REAL := 3.0*Q*T**3*wp**2; -- num z-1
  constant n2 : REAL := 3.0*Q*T**3*wp**2; -- num z-2
  constant n3 : REAL := Q*T**3*wp**2; -- num z-3
  constant d0 : REAL := 4.0*Q*T + 2.0*T**2*wp + Q*T**3*wp**2; -- den z-0
  constant d1 : REAL := -4.0*Q*T + 2.0*T**2*wp + 3.0*Q*T**3*wp**2; -- den z-1
  constant d2 : REAL := -4.0*Q*T - 2.0*T**2*wp + 3.0*Q*T**3*wp**2; -- den z-2
  constant d3 : REAL := 4.0*Q*T - 2.0*T**2*wp + Q*T**3*wp**2; -- den z-3
  constant num : REAL_VECTOR := (n0, n1, n2, n3); -- Numerator array
  constant den : REAL_VECTOR := (d0, d1, d2, d3); -- Denominator array

begin
  EL_output == K * EL_input'ZTF(num, den, T); -- Z-domain transfer function
end architecture z_dmn_2nd;