----------------------------------------------------------------------
-- Friction in Translational: 
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
library IEEE;
  use IEEE.MATH_REAL.all;
  use IEEE.MECHANICAL_SYSTEMS.all;
----------------------------------------------------------------------
entity friction_t is
  generic (
    friction_static : REAL := 1.0;
    friction_kinetic : REAL := 0.5;
    d_viscous : REAL := 0.1;
    velocity_transition : REAL := 1.0e-3;
    k_effective_stuck : REAL := 100.0e3
  );
  port ( terminal tr1, tr2 : translational);
end entity friction_t;
----------------------------------------------------------------------
architecture ideal of friction_t is
  quantity velocity : velocity;
  quantity position across force_total through tr1 to tr2;
  quantity force_d, force_s_k : force := 0.0; -- Viscous damping and static/kinetic forces, respectively[N]
  signal is_stuck : boolean := True;
  signal position_lock : boolean := False;
  signal stuck_position : displacement := 0.0;

begin
  velocity == position'DOT;
  process (velocity'ABOVE(velocity_transition),velocity'ABOVE(-1.0*velocity_transition), force_s_k'ABOVE(friction_static), force_s_k'ABOVE(-1.0*friction_static)) is
  begin
    if not velocity'ABOVE(velocity_transition) and velocity'ABOVE(-1.0*velocity_transition) then
      is_stuck <= True;
      if not position_lock then
        stuck_position <= position;
      end if;
      position_lock <= True;
    end if;

    if force_s_k'ABOVE(friction_static) or not force_s_k'ABOVE(-1.0*friction_static) then
      is_stuck <= False;
      position_lock <= False;
    end if;
  end process;

  if is_stuck use
    force_s_k == k_effective_stuck*(position - stuck_position);
  else
    force_s_k == SIGN(velocity)*friction_kinetic;
  end use;

  force_d == d_viscous*velocity;
  force_total == force_d + force_s_k;
end architecture ideal;