----------------------------------------------------------------------
-- Friction in Rotational : 
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
library IEEE;
  use IEEE.MATH_REAL.all;
  use IEEE.MECHANICAL_SYSTEMS.all;
----------------------------------------------------------------------
entity friction_static_r is
  generic (
    friction_torque_static : REALl := 1.0;
    friction_torque_kinetic : REAL := 0.5;
    d_viscous_rotational : REAL := 0.1;
    w_transition : REAL := 1.0e-3;
    k_eff_rotational_stuck : REAL := 100.0e3
  );
  port ( terminal rot1, rot2 : rotational);
end entity friction_static_r;
----------------------------------------------------------------------
architecture ideal of friction_static_r is
  quantity w : angular_velocity;
  quantity rotation_angle across torque_total through rot1 to rot2;
  quantity torque_d, torque_s_k : torque := 0.0; -- Viscous damping and static/kinetic torques,respectively [N*m]
  signal is_stuck : boolean := True;
  signal angle_lock : boolean := False;
  signal stuck_angle : angle := 0.0;

begin
  w == rotation_angle'DOT;

  process (w'ABOVE(w_transition), w'ABOVE(-1.0*w_transition), torque_s_k'ABOVE(friction_torque_static),torque_s_k'ABOVE(-1.0*friction_torque_static)) is
  begin
    if not w'ABOVE(w_transition) and w'ABOVE(-1.0*w_transition) then
      is_stuck <= True;
      if not angle_lock then
        stuck_angle <= rotation_angle;
      end if;
    end if;
    if torque_s_k'ABOVE(friction_torque_static) or not torque_s_k'ABOVE(-1.0*friction_torque_static) then
      is_stuck <= False;
      angle_lock <= False;
    end if;
  end process;

  if is_stuck use
    torque_s_k == k_eff_rotational_stuck*(rotation_angle - stuck_angle);
  else
    torque_s_k == SIGN(w)*friction_torque_kinetic;
  end use;

  torque_d == d_viscous_rotational*w;
  torque_total == torque_d + torque_s_k;

end architecture ideal;