----------------------------------------------------------------------
-- 2. Driver Model :
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
library IEEE;
    use IEEE.all;
library FUNDAMENTALS_VDA;
library FUNDAMENTALS_JSAE ; 
----------------------------------------------------------------------
entity DriverModel is
  port (
    quantity Q_in_clutch :  in REAL := 0.0;   -- Clutch signal [0/1]
    quantity Q_in_shift :   in REAL := 0.0;   -- Shift position 
    quantity Q_in_vehicle : in REAL := 0.0;   -- Vehicle speed [m/s]
    quantity Q_in_tgt_vehicle : in REAL := 0.0 ;  -- Target speed [km/h]
    quantity Q_out_stp_flag : out REAL;   -- Speed = 0 or not 
    quantity Q_out_brake    : out REAL;   -- Brake stroke [%]
    quantity Q_out_ctrl_cluch : out REAL; -- Clutch stroke [0/1]
    quantity Q_out_ctrl_shift : out REAL; -- Shift position
    quantity Q_out_accel    : out REAL    -- Throttle Stroke [%]
  );
end entity  DriverModel;
------------------------------------
architecture struct of DriverModel is
  quantity Q_clutch, Q_stpflag : REAL ;
  quantity Q_Speed_mps : REAL := 0.0;
  quantity Q_2 : REAL := 0.0;
  quantity Q_3 : REAL := 0.0;
  quantity Q_7 : REAL := 0.0;
  quantity Q_9 : REAL := 0.0;
  quantity Q_11 : REAL := 0.0;
  quantity Q_12 : REAL := 0.0;
  quantity Q_13 : REAL := 0.0;
  quantity Q_14 : REAL := 0.0;
  quantity Q_15 : REAL := 0.0;
  quantity Q_16 : REAL := 0.0;

  quantity comp_jsae1_th, comp_jsae1_vhi, comp_jsae1_vlo : REAL ;
  quantity comp_jsae4_th, comp_jsae4_vhi, comp_jsae4_vlo : REAL ;
  quantity comp_jsae5_th, comp_jsae5_vhi, comp_jsae5_vlo : REAL ;
  quantity comp_jsae6_th, comp_jsae6_vhi, comp_jsae6_vlo : REAL ;
  quantity qsw_jsae4_qinf : REAL ;
  quantity qsw_jsae1_qint : REAL ;
begin 

  -- (1) Shift position
  ---
  q_limiter_vda1 : entity FUNDAMENTALS_VDA.q_limiter_vda(basic)
      generic map (qmax=>5.0, qmin=>1.0)
      port map (q_in=>Q_in_shift, q_out=>Q_out_ctrl_shift);

  -- (2) clutch Position
  ---
  comp_jsae1_th==0.5 ; comp_jsae1_vhi==1.0 ; comp_jsae1_vlo==0.0 ; 
  comp_jsae1 : entity FUNDAMENTALS_JSAE.comp_jsae(behav)
      port map (input=>Q_in_clutch, val=>Q_clutch,
                thres=>comp_jsae1_th, vhi=>comp_jsae1_vhi, vlo=>comp_jsae1_vlo);

  Q_out_ctrl_cluch == Q_clutch ;

  -- (3) Stop state
  ---
  comp_jsae4_th==0.1 ; comp_jsae4_vhi==0.0 ; comp_jsae4_vlo==1.0 ;   
  comp_jsae4 : entity FUNDAMENTALS_JSAE.comp_jsae(behav)
      port map (input=>Q_Speed_mps, val=>Q_stpflag, 
                thres=>comp_jsae4_th, vhi=>comp_jsae4_vhi, vlo=>comp_jsae4_vlo) ;

  Q_out_stp_flag == Q_stpflag ;

  -- (4) Accel Control
  ---
  Q_Speed_mps == Q_in_tgt_vehicle / 3.6 ;

  q_feedback_vda1 : entity FUNDAMENTALS_VDA.q_feedback_vda(basic)
      port map (q_in2=>Q_in_vehicle, q_in1=>Q_Speed_mps, q_out=>Q_13);

  comp_jsae5_th==40.0 ; comp_jsae5_vhi==120.0 ; comp_jsae5_vlo==80.0 ;     
  comp_jsae5 : entity FUNDAMENTALS_JSAE.comp_jsae(behav)
      port map (input=>Q_in_tgt_vehicle, val=>Q_12,
                thres=>comp_jsae5_th, vhi=>comp_jsae5_vhi, vlo=>comp_jsae5_vlo) ; 

  q_mult_vda1 : entity FUNDAMENTALS_VDA.q_mult_vda(basic)
    port map (q_in2=>Q_12, q_in1=>Q_13, q_out=>Q_11);

  q_limiter_vda2 : entity FUNDAMENTALS_VDA.q_limiter_vda(basic)
      generic map (qmax=>100.0, qmin=>0.0)
      port map (q_in=>Q_11, q_out=>Q_9);

  qsw_jsae4_qinf == 0.0 ;
  qswitch_jsae4 : entity FUNDAMENTALS_JSAE.qswitch_jsae(arch_qswitch_jsae)
      port map (ctrl=>Q_clutch, qint=>Q_9, qinf=>0.0, qout=>Q_7);

  q_firstorder_vda1 : entity FUNDAMENTALS_VDA.q_firstorder_vda(basic)
      generic map (t=>0.5, k=>10.0 )
      port map (q_in=>Q_7, q_out=>Q_out_accel);

  -- (5) Brake control 
  ---
  comp_jsae6_th==20.0 ; comp_jsae6_vhi==200.0 ; comp_jsae6_vlo==150.0 ;       
  comp_jsae6 : entity FUNDAMENTALS_JSAE.comp_jsae(behav)
      port map (input=>Q_in_tgt_vehicle, val=>Q_16, 
                thres=>comp_jsae6_th, vhi=>comp_jsae6_vhi, vlo=>comp_jsae6_vlo) ; 

  Q_15 == Q_16 * Q_13 * (-1.0) ;

  q_limiter_vda3 : entity FUNDAMENTALS_VDA.q_limiter_vda(basic)
      generic map (qmax=>100.0, qmin=>0.0)
      port map (q_in=>Q_15, q_out=>Q_14);

  q_firstorder_vda2 : entity FUNDAMENTALS_VDA.q_firstorder_vda(basic)
      generic map (t=>0.5, k=>1.0)
      port map (q_in=>Q_14, q_out=>Q_3);

  qsw_jsae1_qint == 100.0 ;
  qswitch_jsae1 : entity FUNDAMENTALS_JSAE.qswitch_jsae(arch_qswitch_jsae)
      port map (ctrl=>Q_stpflag, qinf=>Q_3, qint=>qsw_jsae1_qint, qout=>Q_2);

  Q_out_brake == Q_2 * 0.01 ; 
  
end architecture struct;
