----------------------------------------------------------------------
-- 3. ECU Model
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
library IEEE;
    use IEEE.all;
library FUNDAMENTALS_VDA;
library FUNDAMENTALS_JSAE ;
----------------------------------------------------------------------
entity ECU_Model is
  port (
    quantity Q_in_tgt_V     : in REAL := 0.0;  -- Target speed [km/h]
    quantity Q_in_ISS_SW    : in REAL := 0.0;  -- IStop flag[0/1]
    quantity Q_in_IdleSpeed : in REAL := 0.0;  -- Idle speed[rpm]
    quantity Q_in_stp_flag  : in REAL := 0.0;  -- Stop flag[0/1]
    quantity Q_in_Kp    : in REAL := 0.01 ;    -- [-]
    quantity Q_in_ACCEL : in REAL := 0.0;  -- Throttle stroke[%]
    quantity Q_in_Ne    : in REAL := 0.0;  -- Engine speed [rpm]
    quantity Q_out_TA      : out REAL;  -- Throttle stroke[%]
    quantity Q_out_FCut_SW : out REAL   --[0/1]
  );
begin
end entity ECU_Model;
----------------------------------------------------------------------
architecture struct of ECU_Model is
  
  quantity Q_2,  Q_3, Q_4,  Q_5, Q_6, Q_7 : REAL := 0.0;
  quantity Q_8,  Q_9, Q_10, Q_11  : REAL := 0.0;
  quantity Q_14, Q_15 : REAL := 0.0;
  quantity Q_17 : REAL := 0.0;

  quantity qsw_jsae2_qinf : REAL ;
  quantity qsw_jsae1_qinf : REAL ;
  quantity qlim_jsae1_ul : REAL ;
begin

  -- (1) ISC control 
  ----
  qnot_jsae1 : entity FUNDAMENTALS_JSAE.qnot_jsae(arch_qnot)
      generic map (td=>0.0)
      port map (yb=>Q_17, xb=>Q_in_ISS_SW);

  qsw_jsae2_qinf==0.0 ;
  qswitch_jsae2 : entity FUNDAMENTALS_JSAE.qswitch_jsae(arch_qswitch_jsae)
      port map (ctrl=>Q_17, qinf=>qsw_jsae2_qinf, qint=>Q_in_IdleSpeed, qout=>Q_15);

  q_feedback_vda1 : entity FUNDAMENTALS_VDA.q_feedback_vda(basic)
      port map (q_out=>Q_11, q_in2=>Q_in_Ne, q_in1=>Q_15 );

  -- (2.1) PI control : P
  ---
  q_proportional_vda1 : entity FUNDAMENTALS_VDA.q_proportional_vda(basic)
      generic map (k=>0.1)
      port map (q_out=>Q_14, q_in=>Q_11);

  q_limiter_vda1 : entity FUNDAMENTALS_VDA.q_limiter_vda(basic)
      generic map (qmax=>10.0, qmin=>0.0)
      port map (q_out=>Q_8, q_in=>Q_14);

  qsw_jsae1_qinf==0.0 ; 
  qswitch_jsae1 : entity FUNDAMENTALS_JSAE.qswitch_jsae(arch_qswitch_jsae)
      port map (ctrl=>Q_in_stp_flag, qinf=>qsw_jsae1_qinf, qint=>Q_in_Kp, qout=>Q_10);
  
  -- (2.2) PI control : I 
  ---
  q_mult_vda1 : entity FUNDAMENTALS_VDA.q_mult_vda(basic)
      port map (q_out=>Q_9, q_in2=>Q_10, q_in1=>Q_11);

  q_limiter_vda2 : entity FUNDAMENTALS_VDA.q_limiter_vda(basic)
      generic map (qmax=>10.0, qmin=>(-10.0) )
      port map (q_out=>Q_5, q_in=>Q_9);

  q_add_vda1 : entity FUNDAMENTALS_VDA.q_add_vda(basic)
      port map (q_out=>Q_3, q_in2=>Q_4, q_in1=>Q_5);

  q_limiter_vda3 : entity FUNDAMENTALS_VDA.q_limiter_vda(basic)
      generic map (qmax=>10.0, qmin=>0.0)
      port map (q_out=>Q_2, q_in=>Q_3 );

  qdelay_jsae1 : entity FUNDAMENTALS_JSAE.qdelay_jsae(behav)
      generic map (td=>0.1)
      port map (input=>Q_2, val=>Q_4 );


  q_add_vda2 : entity FUNDAMENTALS_VDA.q_add_vda(basic)
      port map (q_out=>Q_7, q_in2=>Q_8, q_in1=>Q_2 );

  q_limiter_vda4 : entity FUNDAMENTALS_VDA.q_limiter_vda(basic)
      generic map (qmax=>50.0, qmin=>0.0 )
      port map (q_out=>Q_6, q_in=>Q_7 );

  qlim_jsae1_ul == 100.0 ;
  q_limiter_jsae1 : entity FUNDAMENTALS_JSAE.q_limiter_jsae(basic)
      port map (q_out=>Q_out_TA, q_in=>Q_in_ACCEL, ul=>qlim_jsae1_ul, ll=>Q_6 );

  -- (3) FC control 
  ----
  if(Q_in_ISS_SW > 0.0 and Q_in_tgt_v <= 0.01 and Q_in_tgt_V >= -0.01) use 
    Q_out_FCut_SW == 1.0 ;
  else 
    Q_out_FCut_SW == 0.0 ;
  end use ;

end architecture struct;
        
