----------------------------------------------------------------------
-- 4. Engine Model 
--  4.1 Engine torque map
--  4.2 Engine fuel map
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
library IEEE;
    use IEEE.MECHANICAL_SYSTEMS.all ; 
library FUNDAMENTALS_VDA;
library FUNDAMENTALS_JSAE ;
----------------------------------------------------------------------
entity ENGINE_MODEL is
  port (
    terminal MRV_eng_rot : ROTATIONAL_V;
    quantity Q_in_Fuel_Cut : in REAL := 0.0;  -- fuel cut flag [0/1]
    quantity Q_in_TA       : in REAL := 0.0;  -- Throttle Angle [%]
    quantity Q_out_Ne   : out REAL;        -- Engine speed [rpm]
    quantity Q_out_fuel : out REAL  ;      -- engine fuel [L/h]
    quantity Q_out_power_engine : out REAL -- Engine power [W]
  );
end entity ENGINE_MODEL;
----------------------------------------------------------------------
architecture STRUCT of ENGINE_MODEL is
  terminal n0024 : ROTATIONAL_V;
  quantity Q_2, Q_4, Q_8, Q_9 : REAL := 0.0;
  quantity Q_speed_rpm : REAL := 0.0;
  quantity torque_map : REAL := 0.0;
  quantity speed_engine : REAL := 0.0;
  quantity torque_engine : REAL := 0.0;

  quantity qsw1_qint : REAL ;
  quantity jmass1_j : REAL ;
  quantity friction_static_d : REAL ;
  quantity comp1_th, comp1_vhi, comp1_vlo : REAL ;
  quantity qsw2_qint : REAL ;
begin

  Q_out_power_engine == torque_engine * speed_engine ; 
  Q_speed_rpm == speed_engine * 60.0/3.14/2.0 ;
  Q_out_Ne == Q_speed_rpm ;

  -- (1) Engine torque map
  ---
  engine_torque_tbl1 : entity WORK.engine_torque_tbl(arch_engine_torque_map)
      port map (q_out_eng_trq=>torque_map, q_in_eng_ne=>Q_speed_rpm, q_in_ta=>Q_in_TA );

  -- switch (FuelCut=ON ? 0.0 : Torque)
  ----
  qsw1_qint == 0.0 ; 
  qswitch1 : entity FUNDAMENTALS_JSAE.qswitch_jsae(arch_qswitch_jsae)
      port map (qout=>Q_4, qinf=>torque_map, qint=>qsw1_qint, ctrl=>Q_in_Fuel_Cut );

  -- Mechanical Components 
  ----
  trqsrc1s : entity FUNDAMENTALS_JSAE.trqsrc_rotv_jsae(behav)  -- Engine source
      port map (rot2=>N0024, rot1=>ROTATIONAL_VELOCITY_REF, torque=>Q_4 );

  jmass1_j == 0.14568 ;
  inertia_mass1 : entity FUNDAMENTALS_JSAE.mass_rotv_jsae(behav)
      port map (rot1=>MRV_eng_rot, j=>jmass1_j) ;

  -- Frictions
  ----
  friction_static_d == 0.01 ; 
  friction_static : entity FUNDAMENTALS_JSAE.damp_rotv_jsae(beh)
      port map (r2=>ROTATIONAL_VELOCITY_REF, r1=>MRV_eng_rot, d=>friction_static_d );

  comp1_th == 600.0 ; comp1_vhi == 0.0 ; comp1_vlo==1.0 ; 
  comp_jsae1 : entity FUNDAMENTALS_JSAE.comp_jsae(behav)
      port map (input=>Q_speed_rpm, val=>Q_9, 
                thres=>comp1_th , vhi=>comp1_vhi , vlo=>comp1_vlo) ;

  mul1 : entity FUNDAMENTALS_VDA.q_mult_vda(basic)  --Friction=0 when FuelCut=OFF
      port map (q_out=>Q_8, q_in2=>Q_in_Fuel_Cut, q_in1=>Q_9 );

  friction_dynamic : entity FUNDAMENTALS_JSAE.damp_rotv_jsae(beh)
      port map (r2=>ROTATIONAL_VELOCITY_REF, r1=>MRV_eng_rot, d=>Q_8 );

  -- Engine Torque and Speed sensing
  ----
  torque_sensor1 : entity FUNDAMENTALS_VDA.torque_mrv2q_sensor_vda(simple)
      port map (mrv_2=>MRV_eng_rot, mrv_1=>N0024, q_out=>torque_engine );
  
  NEm : entity FUNDAMENTALS_VDA.angular_velocity2q_sensor_vda(simple)
      port map (mrv_2=>ROTATIONAL_VELOCITY_REF, mrv_1=>MRV_eng_rot, q_out=>speed_engine );


  -- (2) Fuel map table
  ----
  engine_fuel_tbl1 : entity WORK.engine_fuel_tbl(arch_engine_fuel_tbl)
      port map (q_out_eng_fuel=>Q_2, q_in_eng_trq=>Q_4, q_in_eng_ne=>Q_speed_rpm );

  -- switch (FuelCut=ON ? 0.0 : Fuel)
  ----
  qsw2_qint == 0.0 ; 
  qswitch2 : entity FUNDAMENTALS_JSAE.qswitch_jsae(arch_qswitch_jsae)
      port map (qout =>Q_out_fuel, qinf=>Q_2, qint=>qsw2_qint, ctrl=>Q_in_Fuel_Cut );

end architecture STRUCT;
