----------------------------------------------------------------------
-- 5. Clutch Model :
--
-- Copyright (C) 2017 JSAE
-- ----------------------------------------------------------------------------
-- Copyright (C) 2004-2007 VDA/FAT
--
-- This model is a component of the open source library created by the VDA/FAT 
-- working group number 30 and is covered by this license agreement. This model
-- including any updates, modifications, revisions, copies, and documentation
-- are copyrighted works of the VDA/FAT. Use of this model indicates your
-- complete and unconditional acceptance of the terms and conditions set forth
-- in this license agreement.
--  
-- The VDA/FAT grants a non-exclusive license to use, reproduce, modify and
-- distribute this model under the condition, that:
-- (a) no fee or other consideration is charged for any distribution except 
--     compilations distributed in accordance with Section (d) of this license 
--     agreement;
-- (b) the comment text embedded in this model is included verbatim in each
--     copy of this model made or distributed by you, whether or not such
--     version is modified;
-- (c) any modified version must include a conspicuous notice that this model
--     has been modified and the date of modification; and
-- (d) any compilations sold by you that include this model must include a 
--     conspicuous notice that this model is available from the VDA/FAT in 
--     its original form at no charge.
--
-- THIS MODEL IS LICENSED TO YOU "AS IT IS" AND WITH NO WARRANTIES, EXPRESSED
-- OR IMPLIED. THE VDA/FAT AND ALL COMPANIES CONTRIBUTING TO THIS LIBRARY
-- SPECIFICALLY DISCLAIM ALL IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
-- FOR A PARTICULAR PURPOSE. THE VDA/FAT AND ALL COMPANIES ORGANIZED WITHIN
-- MUST NOT HAVE ANY RESPONSIBILITY FOR ANY DAMAGES, FINANCIAL OR LEGAL CLAIMS 
-- WHATEVER.
-- ----------------------------------------------------------------------------
-- Source:
-- clutch.vhd
--
-- Development simulator:
--  
--
-- Tested on: 
-- ------------------------------------------------------------------
-- Version  Simulator      Version   OS    Validator      Date
-- ------------------------------------------------------------------
-- 1.1      
--
-- Modification History:
-- ------------------------------------------------------------------
-- Version  Author            Date           Description
-- ------------------------------------------------------------------
--  1.1    Joachim Haase      2007/03/26     Original Version     
-- 
--
-- ----------------------------------------------------------------------------
-- $Version: 2.0.0 $
-- $Revision: 1746 $
-- $Date: 2007-01-05 16:38:40 +0100 (Fri, 05 Jan 2007) $
-- $Author: haase $
-- ----------------------------------------------------------------------------
	

--/**
--	
-- Clutch.
-- The model describes a simple system level clutch for hybrid electric
-- drive trains. 
-- 
-- @generic   T_CONST    constant input velocity after gear changing
-- @generic   T_SMOOTH   time to smooth input and output velocities 
-- 
-- @port      S_GEAR     gear
-- @port      MRV_IN	 torque input
-- @port      MRV_OUT    torque output
--	
-- @test      bench
-- @reference Mitschke, M.; Wallentowitz, H.:
--            Dynamik der Kraftfahrzeuge. Abschnitt 26.2.
--            Springer-Verlag, 2004 (4. Auflage).
--
--*/					


----------------------------------------------------------------------
library IEEE;
    use IEEE.MECHANICAL_SYSTEMS.all;
----------------------------------------------------------------------
entity CLUTCH is
  port    (
    quantity Q_in_clutch : in REAL ; 
    terminal MRV_IN  : ROTATIONAL_VELOCITY;
    terminal MRV_OUT : ROTATIONAL_VELOCITY
  );           
end entity CLUTCH;
----------------------------------------------------------------------
architecture SYSTEM of  CLUTCH is
    
  quantity ANGULAR_VELOCITY_IN  across TORQUE_IN  through MRV_IN;
  quantity ANGULAR_VELOCITY_OUT across                    MRV_OUT;
  quantity       TORQUE_OUT through ROTATIONAL_VELOCITY_REF to MRV_OUT;
    
begin
    
  if Q_in_clutch = 0.0 use 
    TORQUE_IN  == 0.0;
    TORQUE_OUT == 0.0;
  else
    TORQUE_IN           == TORQUE_OUT;    
    ANGULAR_VELOCITY_IN == ANGULAR_VELOCITY_OUT;
  end use;
    
end architecture SYSTEM;

