----------------------------------------------------------------------
-- 6.1 Gear box model 
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
library IEEE ; 
    use IEEE.MECHANICAL_SYSTEMS.all;
    use IEEE.MATH_REAL.all;
----------------------------------------------------------------------
entity GEARBOX is
  generic (
    TC    : REAL := 0.1 -- rise/fall times
  );
  port    (
    quantity Q_in_gear :in REAL ;
    terminal MRV_IN  : ROTATIONAL_VELOCITY;  -- provides torque
    terminal MRV_OUT : ROTATIONAL_VELOCITY   -- output torque
  );
end entity GEARBOX;
----------------------------------------------------------------------
architecture A0 of  GEARBOX is
  constant RATIO : REAL_VECTOR := (0.0, 3.250, 1.782, 1.172, 0.909, 0.702);    -- gear ratios
  constant ETA   : REAL_VECTOR := (0.96, 0.96, 0.96, 0.96, 0.96, 0.96);    -- efficiency

  quantity ANGULAR_VELOCITY_IN  across TORQUE_IN  through MRV_IN;
  quantity ANGULAR_VELOCITY_OUT across                    MRV_OUT;
  quantity                             TORQUE_OUT through ROTATIONAL_VELOCITY_REF to MRV_OUT;
    
  constant sample_time : TIME := 0.1 * 1 sec;
  signal   S_GEAR  : NATURAL;              -- gear position 

  signal RATIO_GEAR        : REAL :=  0.0;
  signal ETA_GEAR        : REAL :=  0.0;
begin

  process
  begin
    s_gear <= integer(Q_in_gear);
    wait for sample_time;
  end process ;

  RATIO_GEAR <= RATIO(S_GEAR);
  ETA_GEAR <= ETA(S_GEAR);  


  ANGULAR_VELOCITY_IN  == RATIO_GEAR'RAMP(TC)*ANGULAR_VELOCITY_OUT;

  if TORQUE_OUT > -0.1 use
    TORQUE_OUT           == ETA_GEAR'RAMP(TC)*RATIO_GEAR'RAMP(TC)*TORQUE_IN;
  else
    TORQUE_OUT*ETA_GEAR'RAMP(TC) == RATIO_GEAR'RAMP(TC)*TORQUE_IN;
  end use;

end architecture A0;
