----------------------------------------------------------------------
-- 8. Brake Model :
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------	
library IEEE, FUNDAMENTALS_VDA;
    use IEEE.MECHANICAL_SYSTEMS.all;
    use IEEE.MATH_REAL.all;
    use FUNDAMENTALS_VDA.TLU_VDA.all;
----------------------------------------------------------------------
entity BRAKE is
  generic  (
     PEAK    : REAL := 1.0;   -- scalling
     CGEO    : REAL := 1.0;   -- Geometry coef.
     FN_MAX  : REAL := 650.0; -- Max Force[N]
     W_SMALL : REAL := 0.1    -- speed threshold [rad/s]
    );
  port     (
    quantity Q_in_CTRL      : in REAL;  -- Braking power [0 to 1]
    terminal MRV_B  : ROTATIONAL_VELOCITY
    );  

end entity BRAKE;
----------------------------------------------------------------------
architecture BASIC of BRAKE is
  constant W       : REAL_VECTOR    := (0.0, 1.0); -- wheel speed arry
  constant MUE     : REAL_VECTOR    := (0.5, 0.5); -- mue arry
  constant FN_SMALL  : REAL := FN_MAX*1.0E-3;
  constant MUE_0     : REAL := LOOKUP_1D(0.0, W, MUE);
  constant PEAK_FRIC : REAL := PEAK;

  quantity W_FRIC  across TAU_FRIC  through MRV_B;

  quantity FN                : REAL;
  quantity FN_NORMALIZED     : REAL;

begin

  FN_NORMALIZED  == REALMAX(0.0, REALMIN(1.0, Q_in_CTRL));
  FN             == FN_MAX*FN_NORMALIZED;
  
  if not W_FRIC'ABOVE(-W_SMALL) or W_FRIC'ABOVE(W_SMALL) use
     ----
     -- Stopping torque
     ----
     TAU_FRIC   ==  PEAK_FRIC*CGEO*LOOKUP_1D(abs(W_FRIC),W,MUE)*FN*SIGN(W_FRIC);
  else
     ----
     -- Range in small speed
     ----    
     if not FN'ABOVE(FN_SMALL) or FN < FN_SMALL use
        TAU_FRIC == 0.0;
     else
        W_FRIC   == 0.0;
     end use;
  end use;
  
end architecture BASIC;
