----------------------------------------------------------------------
-- 9. Wheel model : 
--
-- Copyright (C) 2017 JSAE
-- ----------------------------------------------------------------------------
-- Copyright (C) 2004-2007 VDA/FAT
--
-- This model is a component of the open source library created by the VDA/FAT 
-- working group number 30 and is covered by this license agreement. This model
-- including any updates, modifications, revisions, copies, and documentation
-- are copyrighted works of the VDA/FAT. Use of this model indicates your
-- complete and unconditional acceptance of the terms and conditions set forth
-- in this license agreement.
--  
-- The VDA/FAT grants a non-exclusive license to use, reproduce, modify and
-- distribute this model under the condition, that:
-- (a) no fee or other consideration is charged for any distribution except 
--     compilations distributed in accordance with Section (d) of this license 
--     agreement;
-- (b) the comment text embedded in this model is included verbatim in each
--     copy of this model made or distributed by you, whether or not such
--     version is modified;
-- (c) any modified version must include a conspicuous notice that this model
--     has been modified and the date of modification; and
-- (d) any compilations sold by you that include this model must include a 
--     conspicuous notice that this model is available from the VDA/FAT in 
--     its original form at no charge.
--
-- THIS MODEL IS LICENSED TO YOU "AS IT IS" AND WITH NO WARRANTIES, EXPRESSED
-- OR IMPLIED. THE VDA/FAT AND ALL COMPANIES CONTRIBUTING TO THIS LIBRARY
-- SPECIFICALLY DISCLAIM ALL IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
-- FOR A PARTICULAR PURPOSE. THE VDA/FAT AND ALL COMPANIES ORGANIZED WITHIN
-- MUST NOT HAVE ANY RESPONSIBILITY FOR ANY DAMAGES, FINANCIAL OR LEGAL CLAIMS 
-- WHATEVER.
-- ----------------------------------------------------------------------------

--  1.1    Joachim Haase      2007/03/26     Original Version     
--  1.2    J. Haase           2007/09/14     Slip behavior included
--
-- ----------------------------------------------------------------------------
-- $Version: 2.0.0 $
-- $Revision: 1746 $
-- $Date: 2007-01-05 16:38:40 +0100 (Fri, 05 Jan 2007) $
-- $Author: haase $
-- ----------------------------------------------------------------------------
	

--/**
--	
-- Wheels.
-- The model describes a simple wheels model for straight ahead
-- car movement. 
-- 
-- @generic   J          moment of intertia of wheel(s) [kg*m^2]
-- @generic   SLIP       slip values
-- @generic   MUE        associated adhesion coefficients
-- @generic   TAU        time constant [s]
-- @generic   V_SMALL    velocity limit (near zero) [m/s]
-- 
-- @port      MRV_IN	 provides torque
-- @port      MTV_OUT    drives vehicle
-- @port      ALPHA      grading angle [deg]
-- @port      STOP_PHASE car stops if TRUE
--	
-- @test      bench
--
-- @reference Ehsani, M.; Gao, Y.; Gay, S. E.; Emadi, A.:
--            Modern Electric, Hybrid Electric, and Fuel Cell Vehicles.
--            CRC Press, 2005.
--
-- @reference Isermann, R.: Fahrdynamik-Regelung.
--            Vieweg-Verlag, 2006.
--
-- @reference Mitschke, M.; Wallentowitz, H.:
--            Dynamik der Kraftfahrzeuge.
--            Springer-Verlag, 2004 (4. Auflage).
--*/					
----------------------------------------------------------------------
library IEEE, FUNDAMENTALS_VDA ;
    use IEEE.MECHANICAL_SYSTEMS.all;
    use IEEE.FUNDAMENTAL_CONSTANTS.all;
    use IEEE.MATH_REAL.all;
    use FUNDAMENTALS_VDA.TLU_VDA.all;
----------------------------------------------------------------------
entity WHEELS is
  generic (
    J       : REAL        := 3.05;        -- moment of inertia of wheels
    TAU     : REAL        := 0.016;      -- time constant for slip
    tire_r  : REAL        := 0.28;        -- radius [m]
    V_SMALL : REAL        := 1.0E-5      -- limit for velocities
  );
  port    (   
    terminal MRV_IN      : ROTATIONAL_VELOCITY;    -- provides torque
    terminal MTV_OUT     : TRANSLATIONAL_VELOCITY  -- drives vehicle
  );

end entity WHEELS;
----------------------------------------------------------------------
architecture SIMPLE of WHEELS is
  constant SLIP    : REAL_VECTOR := (0.0, 0.05, 0.1, 1.0);  -- wheel slip values
  constant MUE     : REAL_VECTOR := (0.0, 1.0,  1.2, 0.8);  -- associated adhesion coefficients 
    
  quantity OMEGA  across TORQUE_IN  through MRV_IN;
  quantity V      across                    MTV_OUT;
  quantity               FORCE_OUT  through TRANSLATIONAL_VELOCITY_REF to MTV_OUT;

  quantity omg_j across trq_j through MRV_IN to ROTATIONAL_VELOCITY_REF ;
  
begin
	
    V        == tire_r * OMEGA;
    FORCE_OUT * tire_r  == TORQUE_IN;  

    trq_j == J * omg_j'dot ; 
       
end architecture SIMPLE;
