----------------------------------------------------------------------
-- 9. Wheel model : 
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
library IEEE, FUNDAMENTALS_VDA ;
    use IEEE.MECHANICAL_SYSTEMS.all;
    use IEEE.FUNDAMENTAL_CONSTANTS.all;
    use IEEE.MATH_REAL.all;
    use FUNDAMENTALS_VDA.TLU_VDA.all;
----------------------------------------------------------------------
entity WHEELS is
  generic (
    J       : REAL        := 3.05;        -- moment of inertia of wheels
    TAU     : REAL        := 0.016;      -- time constant for slip
    tire_r  : REAL        := 0.28;        -- radius [m]
    V_SMALL : REAL        := 1.0E-5      -- limit for velocities
  );
  port    (   
    terminal MRV_IN      : ROTATIONAL_VELOCITY;    -- provides torque
    terminal MTV_OUT     : TRANSLATIONAL_VELOCITY  -- drives vehicle
  );

end entity WHEELS;
----------------------------------------------------------------------
architecture SIMPLE of WHEELS is
  constant SLIP    : REAL_VECTOR := (0.0, 0.05, 0.1, 1.0);  -- wheel slip values
  constant MUE     : REAL_VECTOR := (0.0, 1.0,  1.2, 0.8);  -- associated adhesion coefficients 
    
  quantity OMEGA  across TORQUE_IN  through MRV_IN;
  quantity V      across                    MTV_OUT;
  quantity               FORCE_OUT  through TRANSLATIONAL_VELOCITY_REF to MTV_OUT;

  quantity omg_j across trq_j through MRV_IN to ROTATIONAL_VELOCITY_REF ;
  
begin
	
    V        == tire_r * OMEGA;
    FORCE_OUT * tire_r  == TORQUE_IN;  

    trq_j == J * omg_j'dot ; 
       
end architecture SIMPLE;
