----------------------------------------------------------------------
-- 12. ACG :
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
library IEEE;
use IEEE.ELECTRICAL_SYSTEMS.all ;
use IEEE.MECHANICAL_SYSTEMS.all ;
library FUNDAMENTALS_VDA ;
use FUNDAMENTALS_VDA.all ; 
library FUNDAMENTALS_JSAE ;
use FUNDAMENTALS_JSAE.all ; 
----------------------------------------------------------------------
entity ACG_80A is
  port(
    quantity REG_Voltage: in REAL := 14.0 ; --regulation voltage [v] 
    terminal EL_IN, EL_GND : ELECTRICAL ; 
    terminal MRV_IN, MRV_GND : ROTATIONAL_V 
  ) ; 
end entity ACG_80A ;
----------------------------------------------------------------------
architecture struct of ACG_80A is
  terminal net_15, net_17 : ELECTRICAL;
  terminal n0009,  n0011  : ELECTRICAL;
  terminal n0037 : ROTATIONAL_V;
  quantity Q_v, Q_i, Q_curr, Q_Ilim, Q_omg : REAL := 0.0;

  quantity jmass1_j : REAL ;
begin
  
  q_limiter_jsae1 : entity FUNDAMENTALS_JSAE.q_limiter_jsae(basic)
      port map (q_out =>Q_curr, q_in=>Q_i, ul=>200.0, ll=>0.0 );

  --------
  -- Mechanical Components
  --------  
  trqmap1 : entity WORK.alta_trqmap_80a_150413(beh) ---- (C) Torque Map
      port map (mrv2=>MRV_GND, mrv1=>N0037, v=>Q_v, i=>Q_curr );

  torque_mrv2q_sensor_vda1 : entity FUNDAMENTALS_VDA.torque_mrv2q_sensor_vda(simple)
      port map (mrv_2=>N0037, mrv_1=>MRV_IN);

  jmass1_j == 1.0e-6 ;
  mass_rotv_jsae1 : entity FUNDAMENTALS_JSAE.mass_rotv_jsae(behav)  -- internal inertia
      port map (rot1=>N0037, j=>jmass1_j );

  angular_velocity2q_sensor_vda1 : entity FUNDAMENTALS_VDA.angular_velocity2q_sensor_vda(simple)
      port map (mrv_2=>MRV_GND, mrv_1=>N0037, q_out=>Q_omg);

  
  climit1 : entity WORK.alta_current_limit_80a_140617(arch) ---- (A) Current Limitter
      port map (omg=>Q_omg, Ilim=>Q_Ilim );
  --------
  -- Electrical components
  --------
  acg_emf1 : entity WORK.acg_emf(beh)  ---- (B) EMF calculation
      generic map (ke=>0.133518 )
      port map (EL2=>N0009, EL1=>EL_GND, Vreg=>REG_Voltage, Ilim=>Q_Ilim, omg=>Q_omg );
  
  current2q_sensor_vda1 : entity FUNDAMENTALS_VDA.current2q_sensor_vda(simple)
      port map (el_2=>net_17, el_1=>N0011, q_out=>Q_i);

  voltage2q_sensor_vda1 : entity FUNDAMENTALS_VDA.voltage2q_sensor_vda(simple)
      port map (el_2=>EL_GND, el_1=>N0011, q_out=>Q_v);

  l_vda5 : entity FUNDAMENTALS_VDA.l_vda(basic)
      generic map (l=>0.031*250.0*0.001)
      port map (el2=>net_15, el1=>N0009);

  r_vda5 : entity FUNDAMENTALS_VDA.r_vda(basic)
      generic map (r=>0.0015/1.0	)
      port map (el2=>N0011, el1=>net_15);

  d_jsae5 : entity FUNDAMENTALS_JSAE.d_jsae(equiv)
      generic map (rb=>1.0e-006 , vf=>0.001 , rr=>1000000.0 )
      port map (m=>EL_IN, p=>net_17);
  
end architecture struct;
----------------------------------------------------------------------
