----------------------------------------------------------------------
--  12.2 ACG Back EMF
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
library IEEE; 
use IEEE.ELECTRICAL_SYSTEMS.all ;
----------------------------------------------------------------------
entity ACG_emf is 
  generic (
    Ke : REAL := 0.205509 ; -- back emf coeff.
    Kp : REAL := 1.0        -- regulation control gain =1
  ) ;
  port(
    quantity omg  : in REAL ; -- rotational speed [rad/s]
    quantity Ilim : in REAL ; -- current limitation [A]
    quantity Vreg : in REAL ; --regulate voltage ; 
    terminal EL1, EL2 : ELECTRICAL
  ) ; 
end entity ACG_emf ;
----------------------------------------------------------------------
architecture beh of ACG_emf is 

  quantity v across i through EL1 to EL2 ;

  quantity bemf : REAL ;
  quantity fblim, swlim : REAL ; 
begin

  bemf == Ke*omg ; 
	
  ---
  -- Current limit feedback 
  --
  break on i'above(Ilim) ; 
  if(i >= Ilim) use
    fblim == Kp*(Ilim-i); 
    swlim == 0.0 ; 
  else 
    fblim == 0.0 ; 
    swlim == 1.0 ; 
  end use ; 

  ---
  -- voltage regulation
  --  swlim : for bang-bang control
  ---
  break on bemf'above(Vreg) ;
  if(bemf >= Vreg) use 
     v == -(Vreg + fblim);    -- *swlim;
  else 
     v == -(bemf + fblim);    -- *swlim;
  end use ;

end architecture beh ;
