----------------------------------------------------------------------
-- 13. Battery : (Type: M42)
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
library IEEE;
use IEEE.ELECTRICAL_SYSTEMS.all;
use IEEE.MATH_REAL.all ; 
library FUNDAMENTALS_VDA ;
use FUNDAMENTALS_VDA.all ; 
library FUNDAMENTALS_JSAE ;
use FUNDAMENTALS_JSAE.all ;
----------------------------------------------------------------------
entity N_M42 is
  generic (
    SOC_0 : REAL := 95.0 --initial SOC[%] ;
  ) ;
  port (
   terminal EL_POS, EL_NEG : ELECTRICAL
  );
begin
end entity N_M42;
----------------------------------------------------------------------
architecture struct of N_M42 is
  constant FCC : REAL := 30.0 ; -- Battery rating capacitance [Ah]

  ---- Index args battery_current and soc 
  constant Ibatt_d_map : REAL_VECTOR :=  ( -300.0,   -45.0) ;
  constant soc_map     : REAL_VECTOR :=  ( 90.0, 100.0) ;

  --- param : OCV(current)
  constant ocv_map  : REAL_VECTOR :=  (12.80, 12.88) ; 
  quantity ocv      : REAL ;

  --- param : DC R(soc)
  constant dcir_map : REAL_VECTOR :=  (7.354e-3, 7.177e-3) ; 
  quantity dcir     : REAL ;

  --- param : Discharge R(current, soc) 
  ---              I0     |   I1
  ---       Soc0 | R(0,0)   R(0,1)
  ---       Soc1 | R(1,0)   R(1,1)     
  constant Rd_map   : REAL_VECTOR :=  (1.10e-3,  5.57e-3,
                                       1.50e-3,  6.20e-3) ; 
  quantity Rd        : REAL ;   

  --- Param : Charge R(soc)
  constant Rc_map   : REAL_VECTOR :=  (21.84e-3, 53.44e-3) ; 
  quantity Rc       : REAL ;

  --- Param : Cap1(soc)
  constant Cap1_map  : REAL_VECTOR :=  (364.06, 487.63) ;
  quantity Cap1, Cap2 : REAL ;

  terminal net_1, net_2, net_3, net_5 : ELECTRICAL ;
  terminal net_7, net_8, net_9, net_10: ELECTRICAL ;

  quantity Ibatt : REAL := 0.0;
  quantity soc, sibatt : REAL ; 

BEGIN
  -----------
  -- SoC computation.
  --	soc'dot == 0 leds slower simulation.
  -----------
  if soc >= 100.0 and Ibatt >= 0.0 use
     sibatt == 0.0 ;
  elsif soc <= 0.0 and Ibatt <= 0.0 use
     sibatt == 0.0 ;
  else 
     sibatt == Ibatt ; 
  end use ;
  soc == realmax(Soc_0 + 100.0/(FCC*3600.0)*sibatt'integ, 0.0) ;
  
  am1 : entity FUNDAMENTALS_VDA.current2q_sensor_vda(simple)
      port map ( el_2 => net_9, el_1 =>EL_POS, q_out => Ibatt );

  vm1 : entity FUNDAMENTALS_VDA.voltage2q_sensor_vda(simple)
      port map ( el_2 => EL_NEG, el_1 =>EL_POS);

  ------------
  -- Open Circuit Voltage and DC resistance.
  ------------        
  ocv  == realmax(lookup_1D( soc, soc_map, ocv_map ), 0.0); 
  dcir == realmax(lookup_1D( soc, soc_map, dcir_map), 0.0); 

  ocv1 : entity FUNDAMENTALS_VDA.q2voltage_vda(simple)
      generic map ( default => 1.0 )
      port map ( el_2 =>EL_NEG, el_1 => net_10, q_in => ocv) ;

  r_jsae2 : entity FUNDAMENTALS_JSAE.r_jsae(beh)
      port map ( n => net_10, p => net_5, r => dcir );

   ------------
   -- 
   ------------
  Rc   == realmax(lookup_1D( soc, soc_map, Rc_map ), 0.0); 
  Rd   == realmax(lookup_2D( Ibatt, soc, Ibatt_d_map, soc_map, Rd_map ), 0.0);
  Cap1 == realmax(lookup_1D( soc, soc_map, Cap1_map ), 0.0); 
  Cap2 == Cap1 * 10.0;
        
  ------
  -- Charge 
  ------        
  d_jsae1 : entity FUNDAMENTALS_JSAE.d_jsae(equiv)
      generic map ( rb => 0.0001 , vf => 0.0 , rr => 100000.0 )
      port map ( m => net_1, p => net_9);
        
  r_jsae4 : entity FUNDAMENTALS_JSAE.r_jsae(beh)
      port map ( n => net_3, p => net_1, r => Rc );

  d_jsae2 : entity FUNDAMENTALS_JSAE.d_jsae(equiv)
      generic map ( rb => 0.0001 , vf => 0.0 , rr => 100000.0 )
      port map ( m => net_2, p => net_3);        

  r_jsae1 : entity FUNDAMENTALS_JSAE.r_jsae(beh)
      port map ( n => net_5, p => net_2, r => Rc );
  ------
  -- Discharge
  ------                
  d_jsae3 : entity FUNDAMENTALS_JSAE.d_jsae(equiv)
      generic map ( rb => 0.0001 , vf => 0.0 , rr => 100000.0 )
      port map ( m => net_9, p => net_8);

  r_jsae3 : entity FUNDAMENTALS_JSAE.r_jsae(beh)
      port map ( n => net_3, p => net_8, r => Rd );

  d_jsae4 : entity FUNDAMENTALS_JSAE.d_jsae(equiv)
      generic map ( rb => 0.0001 , vf => 0.0 , rr => 100000.0 )
      port map ( m => net_3, p => net_7);

  r_jsae5 : entity FUNDAMENTALS_JSAE.r_jsae(beh)
      port map ( n => net_5, p => net_7, r => Rd );
  ------
  -- Capacitance
  ------                
  c_jsae1 : entity FUNDAMENTALS_JSAE.c_jsae(beh)
      generic map ( ic => 0.0  )
      port map ( N => net_3, P => net_9, c => Cap1);

  c_jsae2 : entity FUNDAMENTALS_JSAE.c_jsae(beh)
      generic map ( ic => 0.0  )
      port map ( N => net_5, P => net_3, c => Cap2);

end architecture struct;
