----------------------------------------------------------------------
-- 14. Electrical Load : 
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
library IEEE ; 
    use IEEE.ELECTRICAL_SYSTEMS.all ; 
----------------------------------------------------------------------
entity	EL_LOAD is
  generic (
    R_const : REAL := 13.5 ** 2.0 / 75.0 ;
    R_eng   : REAL := 13.5 ** 2.0 / 125.0 ;
    R_stoplamp : REAL := 13.5 ** 2.0 / 30 
  ) ; 
  port (
    quantity Q_in_brake : in REAL := 0.0 ; -- brake signal (1:on/0:off)
    quantity Q_in_Ne    : in REAL := 0.0 ; -- engine speed [rpm]
    terminal EL_P, EL_N : ELECTRICAL
  ) ; 
end entity EL_LOAD;
----------------------------------------------------------------------
architecture arch_el_load of EL_LOAD is

  quantity v_const across i_const through EL_p to EL_n ;
  quantity v_eng   across i_eng   through EL_p to EL_n ; 
  quantity v_brake across i_brake through EL_p to EL_n ;

  signal sw_eng, sw_brake : BOOLEAN ;
  
begin

  -- constant load ; 
  ----
  v_const == R_const * i_const ;

  -- Engine valid load ; 
  ---
  sw_eng <= Q_in_Ne'ABOVE(300.0);
  break on sw_eng ;
    if(sw_eng) use 
      v_eng == R_eng * i_eng ;
    else 
      i_eng == 0.0 ; 
    end use ;

  -- stop lamp load ; 
  ---
  sw_brake <= Q_in_brake'ABOVE(0.5);
  break on sw_brake ;
    if(sw_brake) use 
      v_brake == R_stoplamp * i_brake ;
    else 
      i_brake == 0.0 ; 
    end use ;

end architecture arch_el_load;

