----------------------------------------------------------------------
-- COLUMN model : 
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
library ieee;
library fundamentals_vda;
library fundamentals_jsae;
----------------------------------------------------------------------
entity EPS1_COLUMN is
  generic(
    COULOMB_FRICTION_TORQUE : REAL:=0.05;
    INERTIA : MOMENT_INERTIA:=0.00005;
    VISCOUS_COEFFICIENT : REAL:=0.001
  );
  port(
    terminal COLUMN_LO : ROTATIONAL;
    quantity COLUMN_TRQ : OUT REAL;
    terminal COLUMN_UP : ROTATIONAL
  );
end entity EPS1_COLUMN;
----------------------------------------------------------------------
architecture arch_EPS1_COLUMN of EPS1_COLUMN is

begin
  Q_FROM_TORQUE_R1 : entity FUNDAMENTALS_VDA.TORQUE_MR2Q_SENSOR_VDA(SIMPLE)
    port map (
      Q_OUT => COLUMN_TRQ,
      MR_1 => COLUMN_UP,
      MR_2 => COLUMN_LO
    );

  INERTIA_R1 : entity FUNDAMENTALS_JSAE.INERTIA_R_JSAE(IDEAL)
    generic map (
      J => INERTIA
    )
    port map ( 
      ROT1 => COLUMN_UP 
    );

  FRICTION_STATIC_R1 : entity FUNDAMENTALS_JSAE.FRICTION_STATIC_R(IDEAL)
    generic map (
      D_VISCOUS_ROTATIONAL => VISCOUS_COEFFICIENT,
      FRICTION_TORQUE_KINETIC => COULOMB_FRICTION_TORQUE,
      FRICTION_TORQUE_STATIC => COULOMB_FRICTION_TORQUE
    )
    port map (
      ROT1 => ROTATIONAL_REF,
      ROT2 => COLUMN_UP
    );
end architecture arch_EPS1_COLUMN;