----------------------------------------------------------------------
-- REDUCTIONGEAR model : 
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
lilbrary ieee;
library fundamentals_vda;
library fundamentals_jsae;
----------------------------------------------------------------------
entity EPS1_REDUCTIONGEAR is
  generic(
    COULOMB_FRICTION_TORQUE : REAL:=0.02;
    GEAR_RATIO : REAL:=20.0;
    INERTIA : MOMENT_INERTIA:=0.0001;
    VISCOUS_COEFFICIENT : REAL:=0.005
  );
  port(
    terminal ASSIST : ROTATIONAL;
    quantity ASSIST_TRQ : OUT REAL;
    terminal W_MOTOR : ROTATIONAL_VELOCITY
  );
end entity EPS1_REDUCTIONGEAR;
----------------------------------------------------------------------
architecture arch_EPS1_REDUCTIONGEAR of EPS1_REDUCTIONGEAR is
  terminal W_GEAR: ROTATIONAL_VELOCITY;
  terminal ANG_GEAR: ROTATIONAL;

begin
  INERTIA_R2 : entity FUNDAMENTALS_JSAE.INERTIA_R_JSAE(IDEAL)
    generic map ( 
      J => INERTIA 
    )
    port map ( 
      ROT1 => ASSIST 
    );

  FRICTION_STATIC_R1 : entity FUNDAMENTALS_JSAE.FRICTION_STATIC_R(IDEAL)
    generic map (
      D_VISCOUS_ROTATIONAL => VISCOUS_COEFFICIENT,
      FRICTION_TORQUE_KINETIC => COULOMB_FRICTION_TORQUE,
      FRICTION_TORQUE_STATIC => COULOMB_FRICTION_TORQUE
    )
    port map (
      ROT1 => ASSIST,
      ROT2 => ROTATIONAL_REF
    );

  CONVERT_R_RV6 : entity FUNDAMENTALS_JSAE.CONVERT_R_RV_JSAE(CLOSED_LOOP)
    port map (
      SHAFT_R => ANG_GEAR,
      SHAFT_RV => W_GEAR
    );

  Q_FROM_TORQUE_R2 : entity FUNDAMENTALS_VDA.TORQUE_MR2Q_SENSOR_VDA(SIMPLE)
    port map (
      Q_OUT => ASSIST_TRQ,
      MR_1 => ANG_GEAR,
      MR_2 => ASSIST );

  GEAR_RV1 : entity FUNDAMENTALS_JSAE.GEAR_RV_JSAE(IDEAL)
    generic map ( 
      RATIO => GEAR_RATIO 
    )
    port map (
      ROTV1 => W_GEAR,
      ROTV2 => W_MOTOR
    );
end architecture arch_EPS1_REDUCTIONGEAR;