----------------------------------------------------------------------
-- RACKPINION model : 
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
library ieee;
library fundamentals_vda;
library fundamentals_jsae;
----------------------------------------------------------------------
entity EPS1_RACKPINION is
  generic(
    PINION_COULOMB_FRICTION_TORQUE : REAL:=0.1;
    PINION_INERTIA : MOMENT_INERTIA:=0.00001;
    PINION_PITCH_CIRCLE_RADIUS : REAL:=1.0E-02;
    PINION_VISCOUS_COEFFICIENT : REAL:=0.01;
    RACK_COULOMB_FRICTION_FORCE : REAL:=100.0;
    RACK_END_DAMPER : REAL:=1.0E-09;
    RACK_END_STIFF : REAL:=1.0E+09;
    RACK_MASS : REAL:=3.0;
    RACK_STROKE_M : DISPLACEMENT:=-1.0E-01;
    RACK_STROKE_P : DISPLACEMENT:=1.0E-01;
    RACK_VISCOUS_COEFFICIENT : REAL:=100.0
  );
  port(
    terminal PINION : ROTATIONAL;
    terminal RACK : TRANSLATIONAL
  );
end entity EPS1_RACKPINION;
----------------------------------------------------------------------
architecture arch_EPS1_RACKPINION of EPS1_RACKPINION is

begin
  MASS_T1 : entity FUNDAMENTALS_JSAE.MASS_TR_JSAE(IDEAL)
    generic map ( 
      M => RACK_MASS
    )
    port map ( 
      TRANS1 => RACK 
    );

  INERTIA_R_PINION : entity FUNDAMENTALS_JSAE.INERTIA_R_JSAE(IDEAL)
    generic map ( 
      J => PINION_INERTIA 
    )
    port map ( 
      ROT1 => PINION 
    );

  FRICTION_STATIC_R1 : entity FUNDAMENTALS_JSAE.FRICTION_STATIC_R(IDEAL)
    generic map (
      D_VISCOUS_ROTATIONAL => PINION_VISCOUS_COEFFICIENT,
      FRICTION_TORQUE_KINETIC => PINION_COULOMB_FRICTION_TORQUE,
      FRICTION_TORQUE_STATIC => PINION_COULOMB_FRICTION_TORQUE
    )
    port map ( 
      ROT1 => PINION, ROT2 => ROTATIONAL_REF
    );

  STOP_T1 : entity FUNDAMENTALS_JSAE.STOP_TR_JSAE(IDEAL)
    generic map (
      DAMP_STOP => RACK_END_DAMPER,
      K_STOP => RACK_END_STIFF,
      POS_MAX => RACK_STROKE_P,
      POS_MIN => RACK_STROKE_M
    )
    port map ( 
      TRANS1 => RACK, 
      TRANS2 =>TRANSLATIONAL_REF 
    );

  FRICTION_T1 : entity FUNDAMENTALS_JSAE.FRICTION_T(IDEAL)
    generic map (
      D_VISCOUS => RACK_VISCOUS_COEFFICIENT,
      FRICTION_KINETIC => RACK_COULOMB_FRICTION_FORCE,
      FRICTION_STATIC => RACK_COULOMB_FRICTION_FORCE
    )
    port map ( 
      TR1 => RACK, 
      TR2 => TRANSLATIONAL_REF
    );

  RACK_PINION1 : entity FUNDAMENTALS_JSAE.RACK_PINION(DEFAULT)
    generic map ( 
      RADIUS => PINION_PITCH_CIRCLE_RADIUS
    )
    port map ( 
      PINION => PINION, 
      RACK => RACK 
    );
end architecture arch_EPS1_RACKPINION;