----------------------------------------------------------------------
-- Lowpass Filter JSAE : 
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
library IEEE;
  use ieee.math_real.all;
library fundamentals_vda;
----------------------------------------------------------------------
entity q_LPF_1st_jsae is
  generic (
    Fp : real; -- Pole frequency [Hz]
    K : real := 1.0; -- Filter gain [No Units]
    Fsmp : real := 10.0e3 -- For Z-dmn only: Sample frequency [Hz]
  );
  port (
    quantity input : in real;
    quantity output : out real
  );
end entity q_LPF_1st_jsae;
----------------------------------------------------------------------
-- S Domain Implementation
architecture s_dmn of q_LPF_1st_jsae is
  constant wp : real := math_2_pi*Fp; -- Frequency in Radians
  constant num : real_vector := (wp, 0.0); -- Numerator array
  constant den : real_vector := (wp, 1.0); -- Denominator array

begin
  output == K * input'ltf(num, den); -- Laplace Transfer Function
end architecture s_dmn;
----------------------------------------------------------------------
-- Z Domain Implementation (via bilinear transform, no pre-warping)
architecture z_dmn of q_LPF_1st_jsae is
  constant Tsmp : real := 1.0/Fsmp; -- Sample period
  constant wp : real := fp*math_2_pi; -- Pole in rad/s
  constant numz_0 : real := Tsmp*wp; -- z0 numerator coefficient
  constant numz_1 : real := Tsmp*wp; -- z-1 numerator coefficient
  constant denz_0 : real := Tsmp*wp + 2.0; -- z0 denominator coefficient
  constant denz_1 : real := Tsmp*wp - 2.0; -- z-1 denominator coefficient
  constant num : real_vector := (numz_0, numz_1);
  constant den : real_vector := (denz_0, denz_1);

begin -- ztf
  output == K*input'Ztf(num, den, Tsmp);
end architecture z_dmn;