----------------------------------------------------------------------
-- COLUMN model : 
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
library ieee;
library fundamentals_vda;
library fundamentals_jsae;
----------------------------------------------------------------------
entity Q_PI_REGULATOR_2PH is
  generic(
    K_INT : REAL:=0.0;
    K_LIM : REAL:=1.0;
    K_PROP : REAL:=1.0;
    LIM_HI : REAL:=10.0;
    LIM_LO : REAL:=-10.0;
    LIM_SLOPE : REAL:=1.0E-4
  );
  port(
    quantity FB_D : IN REAL;
    quantity FB_Q : IN REAL;
    quantity IN_D : IN REAL;
    quantity IN_Q : IN REAL;
    quantity OUT_D : OUT REAL;
    quantity OUT_Q : OUT REAL
  );
end entity Q_PI_REGULATOR_2PH;
----------------------------------------------------------------------
architecture arch_Q_PI_REGULATOR_2PH of Q_PI_REGULATOR_2PH is
  quantity GAIN_OUT_Q: REAL;
  quantity ERR_Q: REAL;
  quantity GAIN_OUT_D: REAL;
  quantity ERR_D: REAL;
  quantity INT_OUT_Q: REAL;
  quantity LIM_IN_Q: REAL;
  quantity INT_OUT_D: REAL;
  quantity LIM_IN_D: REAL;

begin
  Q_DIFF_Q : entity FUNDAMENTALS_JSAE.Q_DIFFERENCE_JSAE(BEHAVIORAL)
    port map ( 
      IN1 => IN_Q, 
      IN2 => FB_Q, 
      OUTPUT =>ERR_Q 
    );

  Q_LIM_D : entity FUNDAMENTALS_JSAE.Q_LIMIT_JSAE(SIMPLE)
    generic map (
      K => K_LIM,
      LIMIT_HIGH => LIM_HI,
      LIMIT_LOW => LIM_LO,
      LIMIT_SLOPE => LIM_SLOPE
    )
    port map ( 
      INPUT => LIM_IN_D, 
      OUTPUT => OUT_D 
    );

  Q_SUM_D : entity FUNDAMENTALS_JSAE.Q_SUM_JSAE(BEHAVIORAL)
    port map (
      IN1 => GAIN_OUT_D,
      IN2 => INT_OUT_D,
      OUTPUT => LIM_IN_D
    );

  Q_DIFF_D : entity FUNDAMENTALS_JSAE.Q_DIFFERENCE_JSAE(BEHAVIORAL)
    port map (
      IN1 => IN_D,
      IN2 => FB_D,
      OUTPUT => ERR_D
    );

  Q_GAIN_D : entity FUNDAMENTALS_JSAE.Q_GAIN_JSAE(BEHAVIORAL)
    generic map ( 
      K => K_PROP 
    )
    port map ( 
      INPUT => ERR_D, 
      OUTPUT => GAIN_OUT_D 
    );

  Q_GAIN_Q : entity FUNDAMENTALS_JSAE.Q_GAIN_JSAE(BEHAVIORAL)
    generic map ( 
      K => K_PROP 
    )
    port map ( 
      INPUT => ERR_Q, 
      OUTPUT => GAIN_OUT_Q 
    );

  Q_INTEG_D : entity FUNDAMENTALS_JSAE.Q_INTEG_JSAE(S_DMN)
    generic map ( 
      K => K_INT 
    )
    port map ( 
      INPUT => ERR_D, 
      OUTPUT => INT_OUT_D 
    );

  Q_INTEG_Q : entity FUNDAMENTALS_JSAE.Q_INTEG_JSAE(S_DMN)
    generic map ( 
      K => K_INT 
    )
    port map ( 
      INPUT => ERR_Q, 
      OUTPUT => INT_OUT_Q 
    );

  Q_LIM_Q : entity FUNDAMENTALS_JSAE.Q_LIMIT_JSAE(SIMPLE)
    generic map (
      K => K_LIM,
      LIMIT_HIGH => LIM_HI,
      LIMIT_LOW => LIM_LO,
      LIMIT_SLOPE => LIM_SLOPE
    )
    port map ( 
      INPUT => LIM_IN_Q, 
      OUTPUT => OUT_Q 
    );

  Q_SUM_Q : entity FUNDAMENTALS_JSAE.Q_SUM_JSAE(BEHAVIORAL)
    port map (
      IN1 => GAIN_OUT_Q,
      IN2 => INT_OUT_Q,
      OUTPUT => LIM_IN_Q
    );
end architecture arch_Q_PI_REGULATOR_2PH;
