----------------------------------------------------------------------
-- DC 3 Phase converter : 
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.ELECTRICAL_SYSTEMS.all;
  use IEEE.MATH_REAL.all;
library fundamentals_vda;
  use fundamentals_vda.tlu_vda.all;
----------------------------------------------------------------------
entity Ideal_dc_3ph_converter is
  generic(
    iphase_rms_vector: real_vector:=(-300.0,-257.0,-133.0,-56.0,-30.0,-21.0,-12.3,12.3,21.0,30.0,56.0,133.0,257.0,300.0);
    eff_vector: real_vector:=(0.96,0.96,0.95,0.8,0.6,0.4,0.2,0.2,0.4,0.6,0.8,0.95,0.96,0.96)
  );
  port (
    quantity VA: in real := 0.0;
    quantity VB: in real := 0.0;
    quantity VC: in real := 0.0;
    quantity PWR_AC: out real :=0.0;
    terminal A: ELECTRICAL;
    terminal B: ELECTRICAL;
    terminal C: ELECTRICAL;
    terminal VDC_P: ELECTRICAL;
    terminal VDC_M: ELECTRICAL;
    terminal N: ELECTRICAL
  );
end entity Ideal_dc_3ph_converter;
----------------------------------------------------------------------
architecture default of Ideal_dc_3ph_converter is
  quantity van across ia through A to N;
  quantity vbn across ib through B to N;
  quantity vcn across ic through C to N;
  quantity vbus across ibus through VDC_P to VDC_M;
  quantity power_AC: real:=0.0;
  quantity power_DC: real:=0.0;
  quantity eff: real:=0.25;
  quantity irms,irms2: real:=0.0;
  quantity vratio: real:=1.0;
  constant one_over_sqrt3: real := 1.0/sqrt(3.0);

  function get_vratio(u:real; v:real; vb:real) return
    real is
      variable vbeta,vmag2,vbus2:real;
      variable vratio:real:=1.0;
    begin
      vbeta := (u+2.0*v)*one_over_sqrt3;
      vmag2 := u**2 + vbeta**2;
      vbus2 :=(0.577*vb)**2;
      vratio := sqrt((vbus2+1.0e-6)/(vmag2+1.0e-6));
      return vratio;
  end function get_vratio;
  begin
    if domain=quiescent_domain use
      ibus==0.0;
      ia==0.0;
      ib==0.0;
      ic==0.0;
    elsif not vratio'above(1.0) use
      van==vratio*VA;
      vbn==vratio*VB;
      vcn==vratio*VC;
      power_DC==vbus*ibus;
    else
      van==VA;
      vbn==VB;
      vcn==VC;
      power_DC==vbus*ibus;
    end use;

    vratio==get_vratio(VA,VB,vbus);
    power_AC==van*ia+vbn*ib+vcn*ic;
    irms2==0.667*(ia**2 + ib**2 + ia*ib); --based on Clarke Formula
    irms==sqrt(abs(irms2)+1.0e-6); -- get efficiency from estimated current
    eff == lookup_1d(irms, iphase_rms_vector,eff_vector);
    if power_AC'above(0.0) use -- pos AC means regen
      power_DC==-(power_AC)*eff;
    else -- neg AC means motoring
      power_AC==-(power_DC*eff);
    end use;

    PWR_AC==-power_AC; -- output as a positive number
end architecture default;