----------------------------------------------------------------------
-- STEER_2WHEEL_NEW_VEHICLE_A : 
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
library ieee;
library fundamentals_vda;
library fundamentals_jsae;
----------------------------------------------------------------------
entity STEER_2WHEEL_NEW_VEHICLE_A is
  port(
    quantity FLF : OUT REAL;
    quantity MUF : OUT REAL;
    quantity REAL_TIRE_ANG : IN REAL;
    quantity SPEED : IN REAL
  );
end entity STEER_2WHEEL_NEW_VEHICLE_A;
----------------------------------------------------------------------
architecture arch_STEER_2WHEEL_NEW_VEHICLE_A of STEER_2WHEEL_NEW_VEHICLE_A is
  quantity T2: REAL;
  quantity TAN_DELTA: REAL;
  quantity \$1N15\: REAL;
  terminal \$1N16\: ROTATIONAL_VELOCITY;
  quantity X: REAL;
  quantity V_MPS: REAL;
  quantity V_KMPH: REAL;
  quantity SPD_DEF: REAL;
  quantity PI_TRQ_GARD: REAL;
  quantity \$1N146\: REAL;
  quantity Y: REAL;
  quantity \$1N147\: REAL;
  quantity FX_R: REAL;
  quantity FY_R: REAL;
  quantity \$1N149\: REAL;
  quantity \$1N81\: REAL;
  quantity FX_F: REAL;
  quantity V_X: REAL;
  quantity THETA: REAL;
  quantity FY_F: REAL;
  quantity V_Y: REAL;
  terminal R_WHEEL_TRQ: ROTATIONAL_VELOCITY;
  quantity \$1N151\: REAL;
  quantity PI_TRQ: REAL;
  quantity GAM: REAL;
  quantity ALIGNING_TORQ: REAL;
  quantity \$1N142\: REAL;

begin
  WHEEL2_XY1_R : entity FUNDAMENTALS_JSAE..WHEEL2_XY1(SIMPLE)
    generic map (
      L_A => 1.18,
      L_B => 1.77,
      L_C => -1.77,
      M => 2155.0,
      R => 0.33
    )
    port map (
      MRV => R_WHEEL_TRQ,
      V => V_MPS,
      V_X => V_X,
      V_Y => V_Y,
      GAM => GAM,
      DELTA => \$1N15\,
      V_W => \$1N142\,
      LAMB => \$1N146\,
      MU => \$1N147\,
      F_X => FX_R,
      F_Y => FY_R 
    );

  WHEEL2_XY1_F : entity FUNDAMENTALS_JSAE.WHEEL2_XY1(SIMPLE)
    generic map (
      L_A => 1.77,
      L_B => 1.18,
      L_C => 1.18,
      M => 2155.0,
      R => 0.33
    )
    port map (
      MRV => \$1N16\,
      V => V_MPS,
      V_X => V_X,
      V_Y => V_Y,
      GAM => GAM,
      DELTA => REAL_TIRE_ANG,
      V_W => \$1N149\,
      LAMB => \$1N151\,
      MU => MUF,
      F_X => FX_F,
      F_Y => FY_F
    );

  Q_GAIN2 : entity FUNDAMENTALS_JSAE.Q_GAIN_JSAE(BEHAVIORAL)
    generic map ( 
      K => 1.0 
    )
    port map ( 
      INPUT => FY_F, 
      OUTPUT => FLF 
    );

  TANGENT : entity FUNDAMENTALS_JSAE.TANGENT(BASIC)
    port map (
      DELTA => REAL_TIRE_ANG,
      A => TAN_DELTA
    );

  ALIGNING_TORQUE2 : entity FUNDAMENTALS_JSAE.ALIGNING_TORQUE2(SIMPLE)
    generic map ( 
      BTX0 => 0.051 
    )
    port map (
      DELTA => TAN_DELTA,
      FY => FLF,
      T2 => T2,
      ALIGNING_TORQ => ALIGNING_TORQ
    );

  VEL2POS : entity FUNDAMENTALS_JSAE.VEL2POS(SIMPLE)
    generic map (
      THETA0 => 0.0,
      X0 => 0.0,
      Y0 => 0.0
    )
    port map (
      V_X => V_X,
      V_Y => V_Y,
      GAM => GAM,
      THETA => THETA,
      X => X,
      Y=> Y
    );

  MPS2KMPH2 : entity FUNDAMENTALS_JSAE.MPS2KMPH2(BASIC)
    port map ( 
      MPS => V_MPS, 
      KMPH => V_KMPH 
    );

  VEHICLE_2WHEEL_V_XY1 : entity FUNDAMENTALS_JSAE.VEHICLE_2WHEEL_V_XY1(BASIC)
    generic map ( 
      LF => 1.18, 
      LR => 1.77 
    )
    port map (
      F_X_R => FX_R,
      F_X_F => FX_F,
      F_Y_R => FY_R,
      F_Y_F => FY_F,
      V => V_MPS,
      V_X => V_X,
      V_Y => V_Y,
      GAM => GAM
    );

  Q_PI1 : entity FUNDAMENTALS_VDA.Q_PI_VDA(BASIC)
    generic map ( 
      K => 150.0, 
      T => 20.0 
    )
    port map ( 
      Q_IN => SPD_DEF, 
      Q_OUT => PI_TRQ 
    );

  Q_LIMITER2 : entity FUNDAMENTALS_VDA.Q_LIMITER_VDA(BASIC)
    generic map ( 
      QMAX => 2000.0, 
      QMIN => -2000.0 
    )
    port map ( 
      Q_IN => PI_TRQ, 
      Q_OUT => PI_TRQ_GARD
    );

  U1 : entity FUNDAMENTALS_VDA.Q2TORQUE_MRV_VDA(SIMPLE)
    port map (
      Q_IN => PI_TRQ_GARD,
      MRV_1 => R_WHEEL_TRQ,
      MRV_2 => OPEN
    );

  U2 : entity FUNDAMENTALS_VDA.QDC_VDA(SPICE)
    generic map ( 
      DC => 0.0 
    )
    port map ( 
      Q_OUT => \$1N15\ 
    );

  U3 : entity FUNDAMENTALS_VDA.Q2TORQUE_MRV_VDA(SIMPLE)
    generic map ( 
      DEFAULT => 1.0 
    )
    port map (
      Q_IN => \$1N81\,
      MRV_1 => \$1N16\,
      MRV_2 => OPEN
    );

  U4 : entity FUNDAMENTALS_VDA.QDC_VDA(SPICE)
    generic map ( 
      DC => 0.0 
    )
    port map ( 
      Q_OUT => \$1N81\ 
    );

  Q_FEEDBACK1 : entity FUNDAMENTALS_VDA.Q_FEEDBACK_VDA(BASIC)
    port map (
      Q_IN1 => SPEED,
      Q_IN2 => V_KMPH,
      Q_OUT => SPD_DEF 
    );
end architecture arch_STEER_2WHEEL_NEW_VEHICLE_A;