--///////////////////////////////////////////////////////////////////////////////////////////////////
----------------------------------------------------------------------
--
-- 99. Full vehicle system. Top level schematic
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
--//////////////////////////////////////////////////////////////////////////////////////////////////

library IEEE;
    use IEEE.ELECTRICAL_SYSTEMS.all;
    use IEEE.MECHANICAL_SYSTEMS.all;

library FUNDAMENTALS_VDA ;
library SPICE2VHD ;
library FUNDAMENTALS_JSAE ;

entity Full_SYSTEM is
begin
end entity Full_SYSTEM;

architecture struct of Full_SYSTEM is
	terminal n0068 : ROTATIONAL_V;
	terminal n0260 : ELECTRICAL;
	terminal net_40 : ELECTRICAL;
	terminal net_53 : ROTATIONAL_V;
	terminal net_52 : ROTATIONAL_V;
	terminal n0064 : ROTATIONAL_V;
	terminal n0069 : ROTATIONAL_V;
	terminal n0197 : ROTATIONAL_V;
	terminal n0211 : ELECTRICAL;
	terminal net_50 : TRANSLATIONAL_V;
	terminal n0201 : ELECTRICAL;
	terminal n0130 : ROTATIONAL_V;
	quantity Q_2TA : REAL := 0.0;     -- Throttle angle (ECU->Engine)
	quantity Q_3Fcut : REAL := 0.0;     -- Fuel Cut flag
	quantity Q_4Ne   : REAL := 0.0;     -- Engine Speed [rpm]
	quantity Q_5 : REAL := 0.0;     -- Throttle angle (Drive -> ECU)
	quantity Q_6 : REAL := 0.0;     -- Idle stop flag
	quantity Q_7 : REAL := 0.0;     -- Idling engine speed [rpm]
	quantity Q_8 : REAL := 0.0;     -- Idle stop mode on/off 
	quantity Q_9TgtV : REAL := 0.0;     -- Target vehicle speed [km/hr] (input)
	quantity Q_10Brk : REAL := 0.0;    -- Brake signal 
	quantity Q_11V   : REAL := 0.0;    -- Vehicle speed [m/s]
	quantity Q_12 : REAL := 0.0;    -- Engine fuel [L/h]
	quantity Q_13 : REAL := 0.0;    -- Shift position 
	quantity Q_14 : REAL := 0.0;    -- Clutch signal 
	quantity Q_15 : REAL := 0.0;    -- shift position (input)
	quantity Q_16 : REAL := 0.0;    -- clutch position (input)
	quantity Q_17regV : REAL := 13.0;    -- Alternator reguration voltage
begin

  -----------------------------------------------------------------------
  -- Control system
  -----------------------------------------------------------------------
	controller1 : entity WORK.controller(struct)
                generic map (target_idle_speed=>750.0, IdleStop_on=>1.0)
                port map ( Q_out_shift => Q_15,   Q_out_tgt_v => Q_9TgtV, Q_out_clutch => Q_16,
                           Q_out_iss_flag => Q_8, Q_out_idle_rpm => Q_7 );

	drivermodel1 : entity WORK.drivermodel(struct)
		port map ( Q_out_accel => Q_5, Q_out_ctrl_shift => Q_13, Q_out_ctrl_cluch => Q_14,
                           Q_out_brake => Q_10Brk, Q_out_stp_flag => Q_6,
                           Q_in_tgt_vehicle => Q_9TgtV, Q_in_vehicle => Q_11V, Q_in_shift => Q_15,
                           Q_in_clutch => Q_16 );

	ecu_model1_1 : entity WORK.ecu_model(struct)
		port map ( Q_out_fcut_sw => Q_3Fcut, Q_out_ta => Q_2TA,
                           Q_in_ne => Q_4Ne, Q_in_accel => Q_5, Q_in_kp => 0.01, Q_in_stp_flag => Q_6,
                           Q_in_idlespeed => Q_7, Q_in_iss_sw => Q_8, Q_in_tgt_v => Q_9TgtV );
        
  -----------------------------------------------------------------------
  -- Engines
  -----------------------------------------------------------------------
	engine_model2 : entity WORK.engine_model(struct)
		port map ( MRV_eng_rot => N0068,
                           Q_in_ta => Q_2TA,   Q_in_fuel_cut => Q_3Fcut,
                           Q_out_fuel=>Q_12, Q_out_Ne=>Q_4Ne );

	Te : entity FUNDAMENTALS_VDA.torque_mrv2q_sensor_vda(simple)
		port map ( mrv_2 => N0069, mrv_1 => N0068);

	clutch11 : entity WORK.clutch(system)
		port map ( mrv_out => net_52, mrv_in => N0069, Q_in_clutch => Q_14 );

	gearbox_model1 : entity WORK.gearbox_model(struct)
		generic map ( j => 0.1 )
		port map ( mrv2 => net_53, mrv1 => net_52, Q_in_gear => Q_13 );
        
	DF : entity WORK.df_model(a0)
		generic map ( eta => 0.98 , ratio => 4.058 )
		port map ( mrv_out => N0064, mrv_in => net_53);
        
	brake1 : entity WORK.brake(basic)
		generic map ( w_small => 0.2 , fn_max => 650.0 )
		port map ( MRV_b => N0064, Q_in_ctrl => Q_10Brk );


	wheels4 : entity WORK.wheels(simple)
		generic map ( tire_r => 0.28 , tau => 0.2 , j => 3.05)  -- J=2.8+0.25
		port map ( mtv_out => net_50, mrv_in => N0064);
        
	vehicle1 : entity WORK.vehicle(front_driven)
		generic map ( cd => 0.33 , af => 2.35605 , vehiclemass => 1130.0 )
		port map ( mtv => net_50, Q_out_speed => Q_11V );

  -----------------------------------------------------------------------
  -- ACG (80 A) regulation voltage = 13V
  -----------------------------------------------------------------------
	pulley_model : entity WORK.pulley_model(a0)
		port map ( mrv_out => N0068, mrv_in => N0130);

        Q_17regV == 13.0 ;

	Tc1 : entity FUNDAMENTALS_VDA.torque_mrv2q_sensor_vda(simple)
		port map ( mrv_2 => N0197, mrv_1 => N0130);
        
	acg_80a1 : entity work.acg_80a(struct)
		port map ( MRV_gnd => ROTATIONAL_VELOCITY_REF, MRV_in => N0197,
                           EL_gnd => ELECTRICAL_REF, EL_in => N0211, reg_voltage => Q_17regV );

        
	r2 : entity SPICE2VHD.resistor(spice)
		generic map ( r => 0.002 )
		port map ( n => N0201, p => N0211);

	i_alta : entity FUNDAMENTALS_VDA.current2q_sensor_vda(simple)
		port map ( el_2 => net_40, el_1 => N0201);

  -----------------------------------------------------------------------
  -- Battery (M42)
  -----------------------------------------------------------------------
	n_m42_1 : entity work.n_m42(struct)
		generic map ( soc_0 => 95.0 )
		port map ( EL_neg => electrical_ref, EL_pos => net_40);
        
	v_batt1 : entity FUNDAMENTALS_VDA.voltage2q_sensor_vda(simple)
		port map ( el_2 => electrical_ref, el_1 => net_40);


  -----------------------------------------------------------------------
  -- Electrical Load 
  -----------------------------------------------------------------------        
	el_load1 : entity WORK.el_load(arch_el_load)
		generic map ( r_stoplamp => 6.075 , r_eng => 1.458 , r_const => 2.43 )
		port map ( EL_n => ELECTRICAL_REF, EL_p => net_40,
                           Q_in_ne => Q_4Ne , Q_in_brake => Q_10Brk );

	v_load1 : entity FUNDAMENTALS_VDA.voltage2q_sensor_vda(simple)
		port map ( el_2 => ELECTRICAL_REF, el_1 => net_40);

  -----------------------------------------------------------------------
  -- Starter
  -----------------------------------------------------------------------               
	starter1_1 : entity WORK.starter(struct)
		port map ( EL_in => N0260, Q_in_ctrl => Q_3Fcut );
        
       	Starter_current : entity FUNDAMENTALS_VDA.current2q_sensor_vda(simple)
                port map ( el_2 => N0260, el_1 => net_40);
        
  -----------------------------------------------------------------------
  -- Fuel Comsumption calculation
  -----------------------------------------------------------------------
	fc_calc1 : entity WORK.fc_calc(arch_fc_calc)
		port map ( Q_in_veh_v => Q_11V , Q_in_fuel => Q_12 );



end architecture struct;
----------------------------------------------------------------------
