----------------------------------------------------------------------
-- EPS1_SYSTEM_DRIVE_2WHEEL : 
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
library ieee;
library fundamentals_vda;
library spice2vhd;
library fundamentals_jsae;
----------------------------------------------------------------------
entity EPS1_SYSTEM_DRIVE_2WHEEL is
end entity EPS1_SYSTEM_DRIVE_2WHEEL;
----------------------------------------------------------------------
architecture arch_EPS1_SYSTEM_DRIVE_2WHEEL of EPS1_SYSTEM_DRIVE_2WHEEL is
  quantity FYF: REAL;
  quantity \$1N385\: REAL;
  terminal V_P: ELECTRICAL;
  terminal STEERING: ROTATIONAL;
  quantity SPEED: REAL;
  quantity CURRENT: REAL;
  quantity ASSIST_TRQ: REAL;
  quantity \$1N344\: REAL;
  terminal PINION: ROTATIONAL;
  quantity ALT_CURRENT: REAL;
  quantity PWR_AC: REAL;
  terminal COLUMN: ROTATIONAL;
  terminal V_POS: ELECTRICAL;
  terminal RACK: TRANSLATIONAL;
  terminal ASSIST: ROTATIONAL;
  quantity \$1N427\: REAL;
  quantity COLUMN_TRQ: REAL;
  terminal ALT_IN: ELECTRICAL;
  quantity MUF: REAL;
  quantity INPUT_RAD: REAL;
  quantity TQ_SENS: REAL;
  quantity INPUT_DEG: REAL;
  terminal ENGINE_ROT: ROTATIONAL_VELOCITY;
  quantity \$1N392\: REAL;
  quantity RACK_LOAD: REAL;
  quantity REAL_TIRE_ANG: REAL;
  terminal W_MOTOR: ROTATIONAL_VELOCITY;

begin
  TRQ_ASSIST : entity FUNDAMENTALS_JSAE.VHDL_EPS1_TORQASSIST(ARCH_VHDL_EPS1_TORQASSIST)
    generic map (
      ASSIST_SPEED_GAIN_MAP_X => (0.0,5.0,10.0,100.0,200.0),
      ASSIST_SPEED_GAIN_MAP_Y => (1.0,1.0,0.4,0.2,0.1),
      CTRL_DC_AC_EFFICIENCY_MAP_X => (-300.0,-257.0,-133.0,-56.0,-30.0,-21.0,-12.343: ,12.3,21.0,30.0,56.0,133.0,257.0,300.0),
      CTRL_DC_AC_EFFICIENCY_MAP_Y => (0.96,0.96,0.95,0.8,0.6,0.4,0.4,0.4,0.4,0.6,045: .8,0.95,0.96,0.96)
    )
    port map (
      PWR_AC => PWR_AC,
      SPD => SPEED,
      TQ_SENSE => TQ_SENS,
      VDC_M => ELECTRICAL_REF,
      VDC_P => V_P,
      W_MOTOR => W_MOTOR
    );

  INTERMEDIATE_SHAFT : entity FUNDAMENTALS_JSAE.EPS1_INTERMEDIATESHAFT(ARCH_EPS1_INTERMEDIATESHAFT)
    port map ( 
      INT_LO => PINION, 
      INT_UP => ASSIST 
    );

  RACKPINION : entity FUNDAMENTALS_JSAE.EPS1_RACKPINION(ARCH_EPS1_RACKPINION)
    generic map (
      PINION_INERTIA => 0.00001,
      PINION_PITCH_CIRCLE_RADIUS => 6.9E-3,
      RACK_STROKE_M => -70.0E-3,
      RACK_STROKE_P => 70.0E-3
    )
    port map ( 
      PINION => PINION, 
      RACK => RACK 
    );

  REDUCTION_GEAR : entity FUNDAMENTALS_JSAE.EPS1_REDUCTIONGEAR(ARCH_EPS1_REDUCTIONGEAR)
    generic map ( 
      GEAR_RATIO => 20.0, 
      INERTIA => 0.0001 
    )
    port map (
      ASSIST => ASSIST,
      ASSIST_TRQ => ASSIST_TRQ,
      W_MOTOR => W_MOTOR
    );

  ACG : entity FUNDAMENTALS_JSAE.ACG_80A(STRUCT)
    port map (
      M_GND => ROTATIONAL_VELOCITY_REF,
      E_GND => ELECTRICAL_REF,
      E_IN => ALT_IN,
      M_IN => ENGINE_ROT,
      REG_VOLTAGE => \$1N344\
    );

  N_55B24 : entity FUNDAMENTALS_JSAE.N_55B24(STRUCT)
    generic map ( 
      SOC_0 => 95.0 
    )
    port map ( 
      NEG => ELECTRICAL_REF, 
      POS => V_POS 
    );

  COLUMN_1 : entity FUNDAMENTALS_JSAE.EPS1_COLUMN(ARCH_EPS1_COLUMN)
    port map (
      COLUMN_LO => COLUMN,
      COLUMN_TRQ => COLUMN_TRQ,
      COLUMN_UP => STEERING
    );

  TORQ_SENSOR : entity FUNDAMENTALS_JSAE.EPS1_TORQSENSOR(ARCH_EPS1_TORQSENSOR)
    port map (
      TQ_SENS => TQ_SENS,
      TS_LO => ASSIST,
      TS_UP => COLUMN
    );

  U0 : entity FUNDAMENTALS_VDA.QDC_VDA(SPICE)
    generic map ( 
      DC => 14.01 
    )
    port map ( 
      Q_OUT => \$1N344\ 
    );

  CURRENT_SENSOR1 : entity FUNDAMENTALS_VDA.CURRENT2Q_SENSOR_VDA(SIMPLE)
    port map (
      EL_1 => ALT_IN,
      EL_2 => V_POS,
      Q_OUT => ALT_CURRENT
    );

  CURRENT_SENSOR2 : entity FUNDAMENTALS_VDA.CURRENT2Q_SENSOR_VDA(SIMPLE)
    port map (
      EL_1 => V_POS,
      EL_2 => V_P,
      Q_OUT => CURRENT
    );

  U1 : entity FUNDAMENTALS_VDA.QPULSE_VDA(SPICE)
    generic map (
      PER => 80.0,
      PW => 40.0,
      TF => 1.0,
      TR => 1.0,
      V1 => 0.0,
      V2 => 210.0
    )
    port map ( 
      Q_OUT => \$1N385\ 
    );

  U2 : entity FUNDAMENTALS_VDA.Q2ANGULAR_VELOCITY_VDA(SIMPLE)
    port map (
      Q_IN => \$1N385\,
      MRV_1 => ENGINE_ROT,
      MRV_2 => ROTATIONAL_VELOCITY_REF
    );

  U8 : entity FUNDAMENTALS_VDA.Q2ANGLE_VDA(SIMPLE)
    port map (
      Q_IN => INPUT_RAD,
      MR_1 => STEERING,
      MR_2 => ROTATIONAL_REF
    );

  Q_MULT1 : entity FUNDAMENTALS_VDA.Q_MULT_VDA(BASIC)
    generic map ( 
      GAIN => 3.14/180.0 
    )
    port map (
      Q_IN1 => INPUT_DEG,
      Q_IN2 => \$1N392\,
      Q_OUT => INPUT_RAD
    );

  U9 : entity FUNDAMENTALS_VDA.QDC_VDA(SPICE)
    generic map ( 
      DC => 1.0 
    )
    port map ( 
      Q_OUT => \$1N392\ 
    );

  U11 : entity FUNDAMENTALS_VDA.Q2FORCE_MT_VDA(SIMPLE)
    generic map ( 
      DEFAULT => 1.000000E+000 
    )
    port map ( 
      Q_IN => \$1N427\, 
      MT => RACK 
    );

  Q_TRPF_VDA1 : entity FUNDAMENTALS_VDA.Q_TRPF_VDA(BASIC)
    generic map (
      SEPARATOR => ' ',
      TRFILE => "..\..\DATA\SPEED_P2_20S_VDA.DAT"
    )
    port map ( 
      Q_OUT => SPEED 
    );

  Q_TRPF_VDA2 : entity FUNDAMENTALS_VDA.Q_TRPF_VDA(BASIC)
    generic map (
      SEPARATOR => ' ',
      TRFILE => "..\..\DATA\STEERING_P2_20S_MODIFY_VDA.DAT"
    )
    port map ( 
      Q_OUT => INPUT_DEG 
    );

  Q_GAIN1 : entity FUNDAMENTALS_JSAE.Q_GAIN_JSAE(BEHAVIORAL)
    generic map ( 
      K => -1.0 
    )
    port map (
      INPUT => RACK_LOAD,
      OUTPUT => \$1N427\
    );

  STEER_2WHEEL_NEW_VEHICLE_A : entity FUNDAMENTALS_JSAE.STEER_2WHEEL_NEW_VEHICLE_A(ARCH_STEER_2WHEEL_NEW_VEHICLE_A)
    port map (
      FLF => FYF,
      MUF => MUF,
      REAL_TIRE_ANG => REAL_TIRE_ANG,
      SPEED => SPEED
    );

  PARKING_LOAD_02_MODIFY_A : entity FUNDAMENTALS_JSAE.PARKING_LOAD_02_MODIFY_A(ARCH_PARKING_LOAD_02_MODIFY_A)
    port map (
      FL_F => FYF,
      K_TIRE => REAL_TIRE_ANG,
      RACK => RACK,
      RACK_LOAD => RACK_LOAD,
      SPEED_KMPH => SPEED
    );

  INERTIA_R1 : entity FUNDAMENTALS_JSAE.INERTIA_R_JSAE(IDEAL)
    generic map ( 
      J => 0.02 
    )
    port map ( 
      ROT1 => STEERING 
    );

  FRICTION_STATIC_R : entity FUNDAMENTALS_JSAE.FRICTION_STATIC_R(IDEAL)
    generic map (
      D_VISCOUS_ROTATIONAL => 0.001,
      FRICTION_TORQUE_KINETIC => 0.05,
      FRICTION_TORQUE_STATIC => 0.05
    )
    port map ( 
      ROT1 => ROTATIONAL_REF, 
      ROT2 => STEERING 
    );
end architecture arch_EPS1_SYSTEM_DRIVE_2WHEEL;