--------------------------------------------------
-- DC Motor : 
--
-- Copyright (C) 2017 JSAE
--------------------------------------------------
library IEEE;
use IEEE.MECHANICAL_SYSTEMS.all;
use IEEE.ELECTRICAL_SYSTEMS.all;

entity DCMotor is
  generic (
    R  : RESISTANCE    ; -- winding resistance
    Kt : REAL          ; -- torque constant
    L  : INDUCTANCE    ; -- winding inductance 
    J  : MOMENT_INERTIA; -- value of moment of inertia
    D  : REAL          ; -- damping coefficient
    w0 : REAL          ; -- initial condition for w
    i0 : REAL          ; -- initial condition for i
    theta0 : REAL        -- initial condition for theta
            );
  port (
    terminal EL_p, EL_m : ELECTRICAL;
    terminal MRV_p      : ROTATIONAL_V
        );
end entity DCMotor;

architecture behav of DCMotor is

  quantity v across i through EL_p to EL_m;
  quantity w across torq through MRV_p to rotational_v_ref;
  quantity theta : REAL;

begin

  if domain = quiescent_domain use
    w     == w0;
    i     == i0;
    theta == theta0;
  else
    torq  == -1.0 * Kt * i + J * w'dot + D * w;
    v     == Kt * w + i * R + L * i'dot;
    w     == theta'dot;
  end use;

end architecture behav;