----------------------------------------------------------------------
-- Drive Model (Original) : 
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
-------------------------------------------
-- 2. Driver Model :
--
----------------------------------------------
library IEEE;
use IEEE.all;
library fundamentals_vda;
library fundamentals_jsae ;
----------------------------------------------
entity DriverModel is
  port (
    quantity Q_in_clutch : in REAL := 0.0; -- Clutch signal [0/1]
    quantity Q_in_vehicle : in REAL := 0.0; -- Vehicle speed [m/s]
    quantity Q_in_tgt_vehicle : in REAL := 0.0 ;-- Target speed [km/h]
    quantity Q_out_ctrl_cluch : out REAL; -- Clutch stroke [0/1]
    quantity Q_out_accel : out REAL -- Throttle Stroke [%]
  );
end entity DriverModel;
------------------------------------
architecture struct of DriverModel is
  quantity Q_clutch : REAL ;
  quantity Q_Speed_mps : REAL := 0.0;
  quantity Q_7 : REAL := 0.0;
  quantity Q_9 : REAL := 0.0;
  quantity Q_11 : REAL := 0.0;
  quantity Q_12 : REAL := 0.0;
  quantity Q_13 : REAL := 0.0;
begin


  -- (2) clutch Position
  ---
  comp_jsae1 : entity fundamentals_jsae.comp_jsae(behav)
    port map (input=>Q_in_clutch, val=>Q_clutch,thres=>0.5, vhi=>1.0, vlo=>0.0);

  Q_out_ctrl_cluch == Q_clutch ;


  -- (4) Accel Control
  ---
  q_mult_vda4 : entity fundamentals_vda.q_mult_vda(basic)
    port map (q_in1=>Q_in_tgt_vehicle, q_out=>Q_Speed_mps, q_in2=>1.0/3.6) ;

  q_feedback_vda1 : entity fundamentals_vda.q_feedback_vda(basic)
    port map (q_in2=>Q_in_vehicle, q_in1=>Q_Speed_mps, q_out=>Q_13);

  comp_jsae5 : entity fundamentals_jsae.comp_jsae(behav)
    port map (input=>Q_in_tgt_vehicle, thres=>40.0, vhi=>120.0, vlo=>80.0, val=>Q_12);

  q_mult_vda1 : entity fundamentals_vda.q_mult_vda(basic)
    port map (q_in2=>Q_12, q_in1=>Q_13, q_out=>Q_11);

  q_limiter_vda2 : entity fundamentals_vda.q_limiter_vda(basic)
    generic map (qmax=>100.0, qmin=>0.0)
    port map (q_in=>Q_11, q_out=>Q_9);

  qswitch_jsae4 : entity fundamentals_jsae.qswitch_jsae(arch_qswitch_jsae)
    port map (ctrl=>Q_clutch, qint=>Q_9, qinf=>0.0,qout=>Q_7);

  q_firstorder_vda1 : entity fundamentals_vda.q_firstorder_vda(basic)
    generic map (t=>0.5, k=>10.0 )
    port map (q_in=>Q_7, q_out=>Q_out_accel);


end architecture struct;
