----------------------------------------------------------------------
-- Drive Model (based on the code generated from schematics) : 
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
--------------------------------------------
-- 2. Driver Model :
--
--------------------------------------------
library FUNDAMENTALS_JSAE;
library FUNDAMENTALS_VDA;
--------------------------------------------
entity DriverModel is
  port (
    quantity Q_in_clutch : in REAL := 0.0; -- Clutch signal [0/1]
    quantity Q_in_vehicle : in REAL := 0.0; -- Vehicle speed [m/s]
    quantity Q_in_tgt_vehicle : in REAL := 0.0 ; -- Target speed [km/h]
    quantity Q_out_ctrl_cluch : out REAL; -- Clutch stroke [0/1]
    quantity Q_out_accel : out REAL -- Throttle Stroke [%]
    );
end entity DriverModel;
------------------------------------
architecture struct of DriverModel is
  quantity Q_clutch : REAL ;
  quantity Q_Speed_mps : REAL := 0.0;
  quantity Q_7 : REAL := 0.0;
  quantity Q_9 : REAL := 0.0;
  quantity Q_11 : REAL := 0.0;
  quantity Q_12 : REAL := 0.0;
  quantity Q_13 : REAL := 0.0;
  quantity comp_jsae5_vlo : REAL ;
  quantity comp_jsae5_vhi : REAL ;
  quantity comp_jsae5_th : REAL ;
  quantity q_mult_vda4_q_in2 : REAL ;
  quantity qsw_jsae4_qinf : REAL ;
  quantity comp_jsae1_vhi : REAL ;
  quantity comp_jsae1_vlo : REAL ;
  quantity comp_jsae1_th : REAL ;
begin

  -- (2) clutch Position
  ---
  qdc_vda_comp_jsae1_th : entity FUNDAMENTALS_VDA.qdc_vda(spice)
    generic map (dc => 5.000000e-001,
                 acmag => 0.0, acphase => 0.0)
    port map (q_out => comp_jsae1_th);

  qdc_vda_comp_jsae1_vhi : entity FUNDAMENTALS_VDA.qdc_vda(spice)
    generic map (dc => 1.0,
                 acmag => 0.0, acphase => 0.0)
    port map (q_out => comp_jsae1_vhi);

  qdc_vda_comp_jsae1_vlo : entity FUNDAMENTALS_VDA.qdc_vda(spice)
    generic map (dc => 0.0,
                 acmag => 0.0, acphase => 0.0)
    port map (q_out => comp_jsae1_vlo);

  comp_jsae1 : entity FUNDAMENTALS_JSAE.comp_jsae(behav)
    port map (input => q_in_clutch,
              vlo => comp_jsae1_vlo,
              vhi => comp_jsae1_vhi,
              thres => comp_jsae1_th,
              val => Q_clutch);

  q_proportional_vda_q_clutch : entity FUNDAMENTALS_VDA.qproportional_vda(basic)
    generic map ( k => 1.0)
    port map (q_out => Q_out_ctrl_cluch,
              q_in => Q_clutch);

  -- (4) Accel Control
  ---
  qdc_vda_q_mult_vda4_q_in2 : entity FUNDAMENTALS_VDA.qdc_vda(spice)
    generic map (dc => 1.000/3.600,
                 acmag => 0.0, acphase => 0.0)
    port map (q_out => q_mult_vda4_q_in2);

  q_mult_vda4 : entity FUNDAMENTALS_VDA.q_mult_vda(basic)
    generic map ( gain => 1.0)
    port map (q_out => Q_Speed_mps,
              q_in1 => q_in_tgt_vehicle,
              q_in2 => q_mult_vda4_q_in2);

  q_feedback_vda1 : entity FUNDAMENTALS_VDA.q_feedback_vda(basic)
    port map (q_in2=>Q_in_vehicle, q_in1=>Q_Speed_mps, q_out=>Q_13);

  qdc_vda_comp_jsae5_th : entity FUNDAMENTALS_VDA.qdc_vda(spice)
    generic map (dc => 40.0,
                 acmag => 0.0, acphase => 0.0)
    port map (q_out => comp_jsae5_th);

  qdc_vda_comp_jsae5_vhi : entity FUNDAMENTALS_VDA.qdc_vda(spice)
    generic map (dc => 120.0,
                 acmag => 0.0, acphase => 0.0)
    port map (q_out => comp_jsae5_vhi);

  qdc_vda_comp_jsae5_vlo : entity FUNDAMENTALS_VDA.qdc_vda(spice)
    generic map (dc => 80.0,
                 acmag => 0.0, acphase => 0.0)
    port map (q_out => comp_jsae5_vlo);

  comp_jsae5 : entity FUNDAMENTALS_JSAE.comp_jsae(behav)
    port map (input => q_in_tgt_vehicle,
              vlo => comp_jsae5_vlo,
              vhi => comp_jsae5_vhi,
              thres => comp_jsae5_th,
              val => Q_12);

  q_mult_vda1 : entity FUNDAMENTALS_VDA.q_mult_vda(basic)
    port map (q_in2=>Q_12, q_in1=>Q_13, q_out=>Q_11);

  q_limiter_vda2 : entity FUNDAMENTALS_VDA.q_limiter_vda(basic)
    generic map (qmax=>100.0, qmin=>0.0)
    port map (q_in=>Q_11, q_out=>Q_9);

  qdc_vda_qsw_jsae4_qinf : entity FUNDAMENTALS_VDA.qdc_vda(spice)
    generic map (dc => 0.0,
                acmag => 0.0, acphase => 0.0)
    port map (q_out => qsw_jsae4_qinf);

  qswitch_jsae4 : entity FUNDAMENTALS_JSAE.qswitch_jsae(arch_qswitch_jsae)
    port map (ctrl => Q_clutch,
              qout => Q_7,
              qinf => qsw_jsae4_qinf,
              qint => Q_9);

  q_firstorder_vda1 : entity FUNDAMENTALS_VDA.q_firstorder_vda(basic)
    generic map (t=>0.5, k=>10.0 )
    port map (q_in=>Q_7, q_out=>Q_out_accel);

end struct;
